@@ -15,9 +15,23 @@
 #ifndef CONFIG_H
 #define CONFIG_H 1
 
-#define    CONFIG_FILE           "/etc/dosemu.conf"
-#define    DOSEMU_USERS_FILE     "/etc/dosemu.users"
-#define    DOSEMU_LOGLEVEL_FILE  "/etc/dosemu.loglevel"
+#ifndef __ASM__
+#include "extern.h"
+
+EXTERN char *config_file_path INIT("/etc/dosemu.conf");
+EXTERN char *config_script_path INIT("/var/lib/dosemu/global.conf");
+EXTERN char *dosemu_users_file_path INIT("/etc/dosemu.users");
+EXTERN char *dosemu_loglevel_file_path INIT("/etc/dosemu.loglevel");
+EXTERN char *dosemu_tmpfile_path INIT("/var/run/dosemu.");
+EXTERN char *dosemu_tmpdir_path INIT(".dosemu");
+
+#define    CONFIG_FILE           config_file_path
+#define    CONFIG_SCRIPT         config_script_path
+#define    DOSEMU_USERS_FILE     dosemu_users_file_path
+#define    DOSEMU_LOGLEVEL_FILE  dosemu_loglevel_file_path
+#define    TMPFILE               dosemu_tmpfile_path
+#define    TMPDIR                dosemu_tmpdir_path
+#endif /* not __ASM__ */
 
 /* Do not worry about these */
 #define    LIBSTART  0x20000000
@@ -89,15 +103,13 @@
 #undef X_SUPPORT
 #undef X2_SUPPORT
 #undef X_GRAPHICS
-/* Add if you have X-Keycode problems */
-#undef NEW_KEYCODES
-
-/* Define this to use the new keyboard code */
-#undef NEW_KBD_CODE
 
 /* Define this to use the new port I/O code */
 #undef NEW_PORT_CODE
 
+/* Define this to use the new CMOS code */
+#undef NEW_CMOS
+
 /*
  *  DEFINE this, if you have joystick support in the kernel
  *  or have the joystick module loaded (needing <linux/joystick.h>)
@@ -128,12 +140,18 @@
  */
 #undef RUN_AS_ROOT
 
-/* Define this to use the new X code */
-#undef NEW_X_CODE
+/* Define this to use the new X mouse code */
+#undef NEW_X_MOUSE
 
 /* Define this to use threading code */
 #undef USE_THREADS
 
+/* Define this to use the new interrupt handling code */
+#undef USE_NEW_INT
+
+/* Define this to use the old Slang keyboard code */
+#undef USE_OLD_SLANG_KBD
+
 /* Define this to use the X86 CPU emulator */
 #undef X86_EMULATOR
 
