@@ -7,9 +7,13 @@
  * See /usr/src/linux/drivers/sound/experimental.txt for full details 
  */
 
-/* #define SOUND_FRAG 0x0002000F */
-#define SOUND_FRAG 0x0002000A  /* Optimized for 44Khz Stereo - from Karcher */
-#define SOUND_SIZE 1<<0xA
+#define MAX_NUM_FRAGMENTS       0x20
+
+/* 
+ * milliseconds to buffer sound. It should be high enough to avoid
+ * clicking, but low enough to not delay the sound to much 
+ */
+#define BUFFER_MSECS		60
 
 void linux_sb_mixer_write_setting (int ch, __u8 val);
 __u8 linux_sb_mixer_read_setting(int ch);
