@@ -1,6 +1,3 @@
-#if 0  /* set it 0 for the new INT1A AH=0 code */
-  #define USE_UNIX_TIME_FOR_INT1A_AH0
-#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -23,6 +20,10 @@
 #include "mouse.h"
 #include "disks.h"
 #include "bios.h"
+#ifdef NEW_CMOS
+#include "iodev.h"
+#include "bitops.h"
+#endif
 #include "xms.h"
 #include "int.h"
 #include "dos2linux.h"
@@ -37,14 +38,15 @@
 
 #include "dpmi.h"
 
-#ifdef NEW_KBD_CODE
 #include "keyb_server.h"
-#else
-extern void scan_to_buffer(void);
-#endif
 
 #define DJGPP_HACK	/* AV Feb 97 */
 
+#if X_GRAPHICS
+/* prototype is in X.h -- 1998/03/08 sw */
+int X_change_config(unsigned, void *);
+#endif
+
 /*
    This flag will be set when doing video routines so that special
    access can be given
@@ -93,11 +95,14 @@ void kill_time(long usecs) {
  */
 
 static void default_interrupt(u_char i) {
-  if (d.defint)
-    dbug_printf("int 0x%02x, ax=0x%04x\n", i, LWORD(eax));
+  di_printf("int 0x%02x, ax=0x%04x\n", i, LWORD(eax));
 
   if (!IS_REDIRECTED(i) ||
+#ifndef USE_NEW_INT
       (LWORD(cs) == BIOSSEG && LWORD(eip) == (i * 16 + 2))) {
+#else /* USE_NEW_INT */
+      ((SEGOFF2LINEAR(BIOSSEG, INT_OFF(i)) +2) == SEGOFF2LINEAR(_CS, _IP))) {
+#endif /* USE_NEW_INT */
     g_printf("DEFIVEC: int 0x%02x @ 0x%04x:0x%04x\n", i, ISEG(i), IOFF(i));
 
     /* This is here for old SIGILL's that modify IP */
@@ -107,7 +112,7 @@ static void default_interrupt(u_char i) {
     if ((i != 0x2a) && (i != 0x28))
       g_printf("just an iret 0x%02x\n", i);
   } else
-    run_int(i);
+    real_run_int(i);
 }
 
 static void process_master_boot_record(void)
@@ -239,7 +244,11 @@ static int dos_helper(void)
     if (LO(bx) == 0) {
       if (set_ioperm(0x3b0, 0x3db - 0x3b0, 0))
 	warn("couldn't shut off ioperms\n");
+#ifndef USE_NEW_INT
       SETIVEC(0x10, BIOSSEG, 0x10 * 0x10);	/* restore our old vector */
+#else /* USE_NEW_INT */
+      SETIVEC(0x10, BIOSSEG, INT_OFF(0x10));	/* restore our old vector */
+#endif /* USE_NEW_INT */
       config.vga = 0;
     } else {
       unsigned char *ssp;
@@ -279,7 +288,6 @@ static int dos_helper(void)
       p_dos_str("DPMI-Server Version 0.9 installed\n\n");
     break;
 
-#ifdef NEW_KBD_CODE
    case DOS_HELPER_INSERT_INTO_KEYBUFFER:
       k_printf("KBD: WARNING: outdated keyboard helper fn 6 was called!\n");
       break;
@@ -288,12 +296,6 @@ static int dos_helper(void)
       _AX=get_bios_key();
       k_printf("HELPER: get_bios_key() returned %04x\n",_AX);
       break;
-#else
-   case DOS_HELPER_INSERT_INTO_KEYBUFFER:	/* Do inline int09 insert_into_keybuffer() */
-    k_printf("KBD: Doing INT9 insert_into_keybuffer() bx=0x%04x\n", LWORD(ebx));
-    scan_to_buffer();
-    break;
-#endif
      
   case DOS_HELPER_VIDEO_INIT:
     v_printf("Starting Video initialization\n");
@@ -339,22 +341,12 @@ static int dos_helper(void)
     }
     v_printf("Restore hi vector area\n");
     MEMCPY_2DOS(0x380,save_hi_ints,128);
-    /* The video BIOS just wrote its entry point into the int0x10 vector */
-    if (mice->intdrv) { /* grab int10 back from video card for mouse */
-        void bios_f000(), bios_f000_int10_old();
-        us *ptr = (us*)((BIOSSEG << 4) +
-                        ((long)bios_f000_int10_old - (long)bios_f000));
-        m_printf("ptr is at %p; ptr[0] = %x, ptr[1] = %x\n",ptr,ptr[0],ptr[1]);
-        ptr[0] = IOFF(0x10);
-        ptr[1] = ISEG(0x10);
-        m_printf("after store, ptr[0] = %x, ptr[1] = %x\n",ptr[0],ptr[1]);
-    }
     break;
 
   case DOS_HELPER_GET_DEBUG_STRING:
     /* TRB - handle dynamic debug flags in dos_helper() */
     LWORD(eax) = GetDebugFlagsHelper((char *) (((_regs.es & 0xffff) << 4) +
-					       (_regs.edi & 0xffff)));
+					       (_regs.edi & 0xffff)), 1);
     g_printf("DBG: Get flags\n");
     break;
 
@@ -432,7 +424,7 @@ static int dos_helper(void)
     run_unix_command(SEG_ADR((char *), es, dx));
     break;   
 
-  case DOS_HELPER_0x51:
+  case DOS_HELPER_GET_USER_COMMAND:
     /* Get DOS command from UNIX in es:dx (a null terminated buffer) */
     g_printf("Locating DOS Command\n");
     LWORD(eax) = misc_e6_commandline(SEG_ADR((char *), es, dx));
@@ -450,10 +442,13 @@ static int dos_helper(void)
       LWORD(eax) = 0;
       break;
     }
+    {
+	PRIV_SAVE_AREA
 	enter_priv_off();
 	LWORD(eax) = system(SEG_ADR((char *), es, dx));
 	leave_priv_setting();
 	break;
+    }
 
 #ifdef IPX
   case DOS_HELPER_IPX_CALL:
@@ -470,7 +465,7 @@ static int dos_helper(void)
     break;
 #endif
     case DOS_HELPER_GETCWD:
-        LWORD(eax) = (short)((int)getcwd(SEG_ADR((char *), es, dx), (size_t)LWORD(eax)));
+        LWORD(eax) = (short)((int)getcwd(SEG_ADR((char *), es, dx), (size_t)LWORD(ecx)));
         break;
   case DOS_HELPER_CHDIR:
         LWORD(eax) = chdir(SEG_ADR((char *), es, dx));
@@ -483,6 +478,16 @@ static int dos_helper(void)
   	if (config.cpuemu && !in_dpmi) leave_cpu_emu();
         break;
 #endif
+    case DOS_HELPER_XCONFIG:
+#if X_GRAPHICS
+	if (config.X) {
+		LWORD(eax) = X_change_config((unsigned) LWORD(edx), SEG_ADR((void *), es, bx));
+	} else 
+#endif /* X_GRAPHICS */
+	{
+		_AX = -1;
+	}
+        break;
   case DOS_HELPER_MBR:
     if (LWORD(eax) == 0xfffe) {
       process_master_boot_record();
@@ -509,13 +514,15 @@ static int dos_helper(void)
   return 1;
 }
 
+#ifndef USE_NEW_INT
 static void int08(u_char i)
 {
-  run_int(0x1c);
+  real_run_int(0x1c);
   /* REG(eflags) |= VIF; */
   WRITE_FLAGSE(READ_FLAGSE() | VIF);
   return;
 }
+#endif /* not USE_NEW_INT */
 
 static void int15(u_char i)
 {
@@ -532,7 +539,8 @@ static void int15(u_char i)
     switch (LO(ax))
     {
     case 0x00:			/* giveup timeslice */
-      usleep(INT15_IDLE_USECS);
+      if (config.hogthreshold)
+        usleep(INT15_IDLE_USECS);
       return;
     }
     CARRY;
@@ -554,15 +562,10 @@ static void int15(u_char i)
 	NOCARRY;
 */
     break;
-  case 0x80:			/* default BIOS device open event */
-    LWORD(eax) &= 0x00FF;
-    return;
-  case 0x81:
-    LWORD(eax) &= 0x00FF;
-    return;
-  case 0x82:
-    LWORD(eax) &= 0x00FF;
-    return;
+  case 0x80:		/* default BIOS hook: device open */
+  case 0x81:		/* default BIOS hook: device close */
+  case 0x82:		/* default BIOS hook: program termination */
+    HI(ax) = 0;
   case 0x83:
     h_printf("int 15h event wait:\n");
     show_regs(__FILE__, __LINE__);
@@ -757,41 +760,65 @@ static void int15(u_char i)
   }
 }
 
+#ifdef NEW_CMOS
+void set_ticks(unsigned long new)
+{
+  volatile unsigned long *ticks = BIOS_TICK_ADDR;
+  volatile unsigned char *overflow = TICK_OVERFLOW_ADDR;
+
+  ignore_segv++;
+  *ticks = new;
+  /* A timer read should reset the overflow flag */
+  *overflow = 0;
+  ignore_segv--;
+  h_printf("TICKS: update ticks to %ld\n", new);
+}
+#endif
+
 static void int1a(u_char i)
 {
+#ifdef NEW_CMOS
+  long t;
+#endif
   time_t time_val;
   struct timeval;
   struct timezone;
   struct tm *tm;
 
-#ifdef USE_UNIX_TIME_FOR_INT1A_AH0
-  time_t akt_time;
-  unsigned int test_date;
-#endif
-
   switch (HI(ax)) {
 
-    /* A timer read should reset the overflow flag */
+/*
+--------B-1A00-------------------------------
+INT 1A - TIME - GET SYSTEM TIME
+	AH = 00h
+Return: CX:DX = number of clock ticks since midnight
+	AL = midnight flag, nonzero if midnight passed since time last read
+Notes:	there are approximately 18.2 clock ticks per second, 1800B0h per 24 hrs
+	  (except on Tandy 2000, where the clock runs at 20 ticks per second)
+	IBM and many clone BIOSes set the flag for AL rather than incrementing
+	  it, leading to loss of a day if two consecutive midnights pass
+	  without a request for the time (e.g. if the system is on but idle)
+->	since the midnight flag is cleared, if an application calls this
+->	  function after midnight before DOS does, DOS will not receive the
+->	  midnight flag and will fail to advance the date
+*/
   case 0:			/* read time counter */
-#ifdef USE_UNIX_TIME_FOR_INT1A_AH0
-    time(&akt_time);
-    tm = localtime((time_t *) &akt_time);
-    test_date = tm->tm_year * 10000 + tm->tm_mon * 100 + tm->tm_mday;
-    if ( check_date != test_date ) {
-      start_time = akt_time;
-      check_date = tm->tm_year * 10000 + tm->tm_mon * 100 + tm->tm_mday;
-      g_printf("Over 24hrs forward or backward\n");
+#ifdef NEW_CMOS
+    /* it works because pic_sys_time has a zero reference. It doesn't
+     * account for timer speedups, but any decent DOS program should
+     * call back the original int8 at the 18.2 Hz rate... */
+    last_ticks = sys_base_ticks + usr_delta_ticks + (pic_sys_time >> 16);
+
+    /* has the midnight passed? */
+    if (last_ticks > 1573040) {
       *(u_char *) (TICK_OVERFLOW_ADDR) += 0x1;
+      last_ticks -= 1573040;
     }
-    last_ticks = (tm->tm_hour * 60 * 60 + tm->tm_min * 60 + tm->tm_sec) * 18.206;
     LO(ax) = *(u_char *) (TICK_OVERFLOW_ADDR);
     LWORD(ecx) = (last_ticks >> 16) & 0xffff;
     LWORD(edx) = last_ticks & 0xffff;
-  #if 0
-    g_printf("read timer st:%u ticks:%u act:%u, actdate:%d\n",
-	     start_time, last_ticks, akt_time, tm->tm_mday);
-  #endif
-    set_ticks(last_ticks);
+    g_printf("TIMER: read timer = %lu\n", last_ticks);
+    set_ticks(last_ticks);	/* set_ticks is in rtc.c */
 #else
     ignore_segv++;
     last_ticks = *((unsigned long *)(BIOS_TICK_ADDR));
@@ -802,15 +829,56 @@ static void int1a(u_char i)
     LWORD(edx) = last_ticks & 0xffff;
 #endif
     break;
+
+/*
+--------B-1A01-------------------------------
+INT 1A - TIME - SET SYSTEM TIME
+	AH = 01h
+	CX:DX = number of clock ticks since midnight
+Return: nothing
+Notes:	there are approximately 18.2 clock ticks per second, 1800B0h per 24 hrs
+	  (except on Tandy 2000, where the clock runs at 20 ticks per second)
+	this call resets the midnight-passed flag
+SeeAlso: AH=00h,AH=03h,INT 21/AH=2Dh
+*/
   case 1:			/* write time counter */
+#ifdef NEW_CMOS
+    t = sys_base_ticks + (pic_sys_time >> 16);
+    last_ticks = (LWORD(ecx) << 16) | (LWORD(edx) & 0xffff);
+    usr_delta_ticks = last_ticks - t;
+    set_ticks(last_ticks);
+    g_printf("INT1A: set timer to %#lx\n", last_ticks);
+#else
     last_ticks = (LWORD(ecx) << 16) | (LWORD(edx) & 0xffff);
     set_ticks(last_ticks);
-#ifdef USE_UNIX_TIME_FOR_INT1A_AH0
-    time(&start_time);
-#endif
     g_printf("set timer to %lu \n", last_ticks);
+#endif
     break;
+
+/*
+--------B-1A02-------------------------------
+INT 1A - TIME - GET REAL-TIME CLOCK TIME (AT,XT286,PS)
+	AH = 02h
+Return: CF clear if successful
+	    CH = hour (BCD)
+	    CL = minutes (BCD)
+	    DH = seconds (BCD)
+	    DL = daylight savings flag (00h standard time, 01h daylight time)
+	CF set on error (i.e. clock not running or in middle of update)
+Note:	this function is also supported by the Sperry PC, which predates the
+	  IBM AT; the data is returned in binary rather than BCD on the Sperry,
+	  and DL is always 00h
+SeeAlso: AH=00h,AH=03h,AH=04h,INT 21/AH=2Ch
+*/
   case 2:			/* get time */
+#ifdef NEW_CMOS
+    LOCK_CMOS;
+    HI(cx) = BCD(GET_CMOS(CMOS_HOUR));
+    LO(cx) = BCD(GET_CMOS(CMOS_MIN));
+    HI(dx) = BCD(GET_CMOS(CMOS_SEC));
+    UNLOCK_CMOS;
+    g_printf("INT1A: RTC time %02x:%02x:%02x\n",HI(cx),LO(cx),HI(dx));
+#else
     time(&time_val);
     tm = localtime((time_t *) &time_val);
     g_printf("get time %d:%02d:%02d\n", tm->tm_hour, tm->tm_min, tm->tm_sec);
@@ -829,18 +897,27 @@ static void int1a(u_char i)
     HI(dx) |= tm->tm_sec << 4;
     /* LO(dx) = tm->tm_isdst; */
     /* REG(eflags) &= ~CF; */
+#endif
     NOCARRY;
     break;
+
+/*
+--------B-1A04-------------------------------
+INT 1A - TIME - GET REAL-TIME CLOCK DATE (AT,XT286,PS)
+	AH = 04h
+Return: CF clear if successful
+	    CH = century (BCD)
+	    CL = year (BCD)
+	    DH = month (BCD)
+	    DL = day (BCD)
+	CF set on error
+SeeAlso: AH=02h,AH=04h"Sperry",AH=05h,INT 21/AH=2Ah,INT 4B/AH=02h"TI"
+*/
   case 4:			/* get date */
     time(&time_val);
     tm = localtime((time_t *) &time_val);
     tm->tm_year += 1900;
     tm->tm_mon++;
-    g_printf("get date %02d.%02d.%04d\n", tm->tm_mday, tm->tm_mon, tm->tm_year);
-#if 0
-    gettimeofday(&tp, &tzp);
-    ticks = tp.tv_sec - (tzp.tz_minuteswest * 60);
-#endif
     LWORD(ecx) = tm->tm_year % 10;
     tm->tm_year /= 10;
     LWORD(ecx) |= (tm->tm_year % 10) << 4;
@@ -855,14 +932,86 @@ static void int1a(u_char i)
     tm->tm_mon /= 10;
     HI(dx) |= tm->tm_mon << 4;
     /* REG(eflags) &= ~CF; */
+    g_printf("INT1A: RTC date %04x%02x%02x (DOS format)\n", _CX, _DH, _DL);
     NOCARRY;
     break;
 
+/*
+--------B-1A03-------------------------------
+INT 1A - TIME - SET REAL-TIME CLOCK TIME (AT,XT286,PS)
+	AH = 03h
+	CH = hour (BCD)
+	CL = minutes (BCD)
+	DH = seconds (BCD)
+	DL = daylight savings flag (00h standard time, 01h daylight time)
+Return: nothing
+Note:	this function is also supported by the Sperry PC, which predates the
+	  IBM AT; the data is specified in binary rather than BCD on the
+	  Sperry, and the value of DL is ignored
+--------B-1A05-------------------------------
+INT 1A - TIME - SET REAL-TIME CLOCK DATE (AT,XT286,PS)
+	AH = 05h
+	CH = century (BCD)
+	CL = year (BCD)
+	DH = month (BCD)
+	DL = day (BCD)
+Return: nothing
+*/
   case 3:			/* set time */
   case 5:			/* set date */
-    g_printf("WARNING: timer: can't set time/date\n");
+    g_printf("INT1A: RTC: can't set time/date\n");
     break;
 
+#ifdef NEW_CMOS
+  /* Notes: the alarm occurs every 24 hours until turned off, invoking INT 4A
+  	each time the BIOS does not check for invalid values for the time, so
+  	the CMOS clock chip's "don't care" setting (any values between C0h
+  	and FFh) may be used for any or all three parts.  For example, to
+  	create an alarm once a minute, every minute, call with CH=FFh, CL=FFh,
+  	and DH=00h. (R.Brown)
+   */
+  case 6:			/* set alarm */
+    {
+      int stb;
+      unsigned char h,m,s;
+
+      LOCK_CMOS;
+      stb=GET_CMOS(CMOS_STATUSB);
+      if (stb&0x20) {
+        CARRY;
+      } else {
+	/* bit 2 of register 0xb set=binary mode, clear=BCD mode */
+	if (stb & 4) {
+          SET_CMOS(CMOS_HOURALRM, (h=_CH));
+          SET_CMOS(CMOS_MINALRM, (m=_CL));
+          SET_CMOS(CMOS_SECALRM, (s=_DH));
+	}
+	else {
+          SET_CMOS(CMOS_HOURALRM, (h=BIN(_CH)));
+          SET_CMOS(CMOS_MINALRM, (m=BIN(_CL)));
+          SET_CMOS(CMOS_SECALRM, (s=BIN(_DH)));
+	}
+        r_printf("RTC: set alarm to %02d:%02d:%02d\n",h,m,s);  /* BIN! */
+        /* This has been VERIFIED on an AMI BIOS -- AV */
+        SET_CMOS(CMOS_STATUSB, stb|0x20);
+        clear_bit (PIC_IRQ8, &pic1_imr);
+        NOCARRY;
+      }
+      UNLOCK_CMOS;
+      break;
+    }
+
+  case 7:			/* clear alarm but NOT PIC mask */
+    /* This has been VERIFIED on an AMI BIOS -- AV */
+    LOCK_CMOS;
+    SET_CMOS(CMOS_STATUSB, GET_CMOS(CMOS_STATUSB)&~0x20);
+    UNLOCK_CMOS;
+#ifdef NEW_PIC
+    pic_untrigger(PIC_IRQ8);
+#endif
+    break;
+#endif
+ 
   case 0xb1:			/* Intel PCI BIOS v 2.0c */
     switch (LO(ax)) {
       case 1: case 0x81:	/* installation check */
@@ -887,12 +1036,22 @@ static void int1a(u_char i)
 }
 
 /* ========================================================================= */
-/* note that the emulation herein may cause problems with programs
+/*
+ * DANG_BEGIN_FUNCTION ms_dos
+ *
+ * int0x21 call
+ *
+ * we trap this for two functions: simulating the EMMXXXX0 device and
+ * fudging the CONFIG.XXX and AUTOEXEC.XXX bootup files.
+ *
+ * note that the emulation herein may cause problems with programs
  * that like to take control of certain int 21h functions, or that
  * change functions that the true int 21h functions use.  An example
  * of the latter is ANSI.SYS, which changes int 10h, and int 21h
  * uses int 10h.  for the moment, ANSI.SYS won't work anyway, so it's
  * no problem.
+ *
+ * DANG_END_FUNCTION
  */
 /* XXX - MAJOR HACK!!! this is bad bad wrong.  But it'll probably work
  * unless someone puts "files=200" in his/her config.sys
@@ -907,9 +1066,6 @@ static void int1a(u_char i)
 
 static int ms_dos(int nr)
 {
-  /* we trap this for two functions: simulating the EMMXXXX0 device and
-   * fudging the CONFIG.XXX and AUTOEXEC.XXX bootup files.
-   */
   switch (nr) {
   case 0x3d:       /* DOS handle open */
   {
@@ -928,10 +1084,9 @@ static int ms_dos(int nr)
  ext_fix(config.emuini);
       sprintf(ptr+2, "\\WINDOWS\\SYSTEM.%s", config.emuini);
       d_printf("DISK: Substituted %s for system.ini\n", ptr);
-#ifndef INTERNAL_EMS
     }
-#else
-    } else if (config.ems_size && !strncmp(ptr, "EMMXXXX0", 8)) {
+#ifdef INTERNAL_EMS
+    else if (config.ems_size && !strncmp(ptr, "EMMXXXX0", 8)) {
       E_printf("EMS: opened EMM file!\n");
       LWORD(eax) = EMM_FILE_HANDLE;
       NOCARRY;
@@ -984,19 +1139,22 @@ static int ms_dos(int nr)
     return 0;
 
   default:
-    g_printf("INT21 (0x%02x):  we shouldn't be here! ax=0x%04x, bx=0x%04x\n",
+    if (!in_dpmi)
+      g_printf("INT21 (0x%02x):  we shouldn't be here! ax=0x%04x, bx=0x%04x\n",
 	     nr, LWORD(eax), LWORD(ebx));
     return 0;
   }
   return 1;
 }
 
-void
-run_int(int i)
+/* ========================================================================= */
+
+void real_run_int(int i)
 {
   unsigned char *ssp;
   unsigned long sp;
 
+#ifndef USE_NEW_INT
   /* ssp = (unsigned char *)(REG(ss)<<4); */
   ssp = (unsigned char *)(READ_SEG_REG(ss)<<4);
   sp = (unsigned long) LWORD(esp);
@@ -1010,15 +1168,124 @@ run_int(int i)
   WRITE_SEG_REG(cs, ((us *) 0)[(i << 1) + 1]);
   LWORD(eip) = ((us *) 0)[i << 1];
 
+#else /* USE_NEW_INT */
+  ssp = (unsigned char *)(_SS<<4);
+  sp = (unsigned long) _SP;
+
+  pushw(ssp, sp, read_FLAGS());
+  pushw(ssp, sp, _CS);
+  pushw(ssp, sp, _IP);
+  _SP -= 6;
+  _CS = ISEG(i);
+  _IP = IOFF(i);
+#endif /* USE_NEW_INT */
+
   /* clear TF (trap flag, singlestep), VIF/IF (interrupt flag), and
    * NT (nested task) bits of EFLAGS
    * NOTE: IF-flag only, because we are not sure that we will test it in
    *       some of our own software (...we all are human beings)
    *       For vm86() 'VIF' is the candidate to reset in order to do CLI !
    */
+#ifndef USE_NEW_INT
   WRITE_FLAGSE(READ_FLAGSE() & ~(VIF | TF | IF | NT));
+#else /* USE_NEW_INT */
+  clear_TF();
+  clear_NT();
+  clear_IF();
+#endif /* USE_NEW_INT */
 }
 
+#ifdef USE_NEW_INT
+/* DANG_BEGIN_FUNCTION run_caller_func(i, from_int)
+ *
+ * This function runs the specified caller function in response to an
+ * int instruction.  Where i is the interrupt function to execute and
+ * from_int specifies if we are comming directly from an int
+ * instruction.
+ *
+ * This function runs the instruction with the following model _CS:_IP is the
+ * address to start executing at after the caller function terminates, and
+ * _EFLAGS are the flags to use after termination.  For the simple case of an
+ * int instruction this is easy.  _CS:_IP = retCS:retIP and _FLAGS = retFLAGS
+ * as well equally the current values (retIP = curIP +2 technically).
+ *
+ * However if the function is called (from dos) by simulating an int instruction
+ * (something that is common with chained interrupt vectors) 
+ * _CS:_IP = BIOS_SEG:HLT_OFF(i) and _FLAGS = curFLAGS 
+ * while retCS, retIP, and retFlags are on the stack.  These I pop and place in
+ * the appropriate registers.  
+ *
+ * This functions actions certainly correct for functions executing an int/iret
+ * discipline.  And almost certianly correct for functions executing an
+ * int/retf#2 discipline (with flag changes), as an iret is always possilbe.
+ * However functions like dos int 0x25 and 0x26 executing with a int/retf will
+ * not be handled correctlty by this function and if you need to handle them
+ * inside dosemu use a halt handler instead.
+ *
+ * Finally there is a possible trouble spot lurking in this code.  Interrupts
+ * are only implicitly disabled when it calls the caller function, so if for
+ * some reason the main loop would be entered before the caller function returns
+ * wrong code may execute if the retFLAGS have interrupts enabled!  
+ *
+ * This is only a real handicap for sequences of dosemu code execute for long
+ * periods of time as we try to improve timer response and prevent signal queue
+ * overflows! -- EB 10 March 1997
+ *
+ * Grumble do to code that executes before interrupts, and the
+ * semantics of default_interupt, I can't implement this function as I
+ * would like.  In the tricky case of being called from dos by
+ * simulating an int instruction, I must leave retCS, retIP, on the
+ * stack.  But I can safely read retFlags so I do.  
+ * I pop retCS, and retIP just before returning to dos, as well as
+ * dropping the stack slot  that held retFlags.
+ *
+ * This improves consistency of interrupt handling, but not quite as
+ * much as if I could handle it the way I would like.
+ * -- EB 30 Nov 1997
+ *
+ * DANG_END_FUNCTION
+ */
+
+static void run_caller_func(int i, Boolean from_int)
+{
+	void (*caller_function)(int i);
+	ifprintf((d.dos > 1), "DO_INT0x%02x: Using caller_function()\n", i);
+
+#if 0	 /* This causes problems with default_interrupt disable this for now
+          * --EB 13 March 1997 
+	  */
+
+	if (!from_int) {
+		_IP = POPW(_SS, _SP);
+		_CS = POPW(_SS, _SP);
+		set_FLAGS(POPW(_SS, _SP));
+	}
+#else
+	/* Do to a misfeature I must leave retCS & retIP on the stack, but I
+	 * can still read retFlags.
+	 */
+	if (!from_int) {
+		unsigned char *ssp = (unsigned char *)(_SS<<4);
+		unsigned long sp = (Bit16u)(_SP +4);
+		set_FLAGS(popw(ssp, sp));
+	}
+#endif
+	caller_function = interrupt_function[i];
+	if (caller_function) {
+
+		caller_function(i);
+	}
+	if (!from_int) {
+		unsigned char *ssp = (unsigned char *)(_SS<<4);
+		unsigned long sp = _SP;
+		_SP += 6;
+		_IP = popw(ssp, sp);
+		_CS = popw(ssp, sp);
+		/* set_FLAGS(popw(ssp, sp)); */
+	}
+}
+#endif /* USE_NEW_INT */
+
 int can_revector(int i)
 {
 /* here's sort of a guideline:
@@ -1114,6 +1381,7 @@ static void int05(u_char i) {
     return;
 }
 
+#ifndef USE_NEW_INT
 void int_a_b_c_d_e_f(u_char i) {
     g_printf("IRQ->interrupt %x\n", i);
     show_regs(__FILE__, __LINE__);
@@ -1124,9 +1392,10 @@ void int_a_b_c_d_e_f(u_char i) {
 /* IRQ1, keyb data ready */
 static void int09(u_char i) {
     fprintf(stderr, "IRQ->interrupt %x\n", i);
-    run_int(0x09);
+    real_run_int(0x09);
     return;
 }
+#endif /* not USE_NEW_INT */
 
 /* CONFIGURATION */
 static void int11(u_char i) {
@@ -1140,11 +1409,13 @@ static void int12(u_char i) {
     return;
 }
 
+#ifndef USE_NEW_INT
 /* KEYBOARD */
 static void int16(u_char i) {
-    run_int(0x16);
+    real_run_int(0x16);
     return;
 }
+#endif /* not USE_NEW_INT */
 
 /* BASIC */
 static void int18(u_char i) {
@@ -1158,6 +1429,9 @@ static void int19(u_char i) {
 
 /* MS-DOS */
 static void int21(u_char i) {
+  ds_printf("INT21 at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
+       LWORD(cs), LWORD(eip),
+       LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
   if (!ms_dos(HI(ax)))
     default_interrupt(i);
 }
@@ -1173,28 +1447,61 @@ static void int23(u_char i)
   if (in_dpmi)
 	NOCARRY;
   else
-	run_int(0x23);
+	real_run_int(0x23);
   return; 
 }
 #endif
 
+static void mouse_post_boot(void)
+{
+	void bios_f000(), bios_f000_int10_old();
+	us *ptr;
+	/* This is needed here to revectoring the interrupt, after dos
+	 * has revectored it. --EB 1 Nov 1997 */
+
+	/* This code is dupped for now in base/mouse/mouse.c - JES 96/10/20 */
+	#define Mouse_INT       (0x33 * 16)
+	SETIVEC(0x33, Mouse_SEG, Mouse_INT);
+#ifndef USE_NEW_INT
+      	#define Mouse_INT74     (0x74 * 16)
+	SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
+#else /* USE_NEW_INT */
+#if 0
+	SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
+#endif
+
+#endif /* USE_NEW_INT */
+      
+	/* grab int10 back from video card for mouse */
+        ptr = (us*)((BIOSSEG << 4) +
+		    ((long)bios_f000_int10_old - (long)bios_f000));
+        m_printf("ptr is at %p; ptr[0] = %x, ptr[1] = %x\n",ptr,ptr[0],ptr[1]);
+        ptr[0] = IOFF(0x10);
+        ptr[1] = ISEG(0x10);
+        m_printf("after store, ptr[0] = %x, ptr[1] = %x\n",ptr[0],ptr[1]);
+#if defined(USE_NEW_INT) || (INT10_WATCHER_SEG != BIOSSEG)
+	 /* Otherwise this isn't safe */
+	SETIVEC(0x10, INT10_WATCHER_SEG, INT10_WATCHER_OFF);
+#endif
+}
+
+static void dos_post_boot(void)
+{
+    if (!config.keybint && config.console_keyb) {
+      /* revector int9, so dos doesn't play with the keybuffer */
+      k_printf("revectoring int9 away from dos\n");
+      SETIVEC(0x9, BIOSSEG, 16 * 0x8 + 2);  /* point to the IRET before INT9 */
+    }
+    if (mice->intdrv) mouse_post_boot();
+
+}
+
 /* KEYBOARD BUSY LOOP */
 static void int28(u_char i) {
   static int first = 1;
   if (first) {
     first = 0;
-    if (!config.keybint && config.console_keyb) {
-      /* revector int9, so dos doesn't play with the keybuffer */
-      k_printf("revectoring int9 away from dos\n");
-      SETIVEC(0x9, BIOSSEG, 16 * 0x8 + 2);  /* point to the IRET before INT9 */
-    }
-    if (mice->intdrv) {
-      /* This code is dupped for now in base/mouse/mouse.c - JES 96/10/20 */
-      #define Mouse_INT       (0x33 * 16)
-      #define Mouse_INT74     (0x74 * 16)
-      SETIVEC(0x33, Mouse_SEG, Mouse_INT);
-      SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
-    }
+    dos_post_boot();
   }
 
   if (config.hogthreshold) {
@@ -1219,7 +1526,8 @@ static void int2f(u_char i)
 {
   switch (LWORD(eax)) {
   case INT2F_IDLE_MAGIC:   /* magic "give up time slice" value */
-    usleep(INT2F_IDLE_USECS);
+    if (config.hogthreshold)
+      usleep(INT2F_IDLE_USECS);
     LWORD(eax) = 0;
     return;
 
@@ -1327,7 +1635,7 @@ static void int2f(u_char i)
     return;
   }
 
-  if IS_REDIRECTED(i)
+  if (IS_REDIRECTED(i))
     default_interrupt(i);
 }
 
@@ -1346,6 +1654,9 @@ static void int33(u_char i) {
  * reschedules dosemu it will then start executing the real mode mouse handler). :-( 
  * Do we need/have we got post_interrupt (IRET) handlers? 
  */
+#ifdef USE_NEW_INT
+/* We have post_interrupt handlers in dpmi --EB 28 Oct 1997 */
+#endif /* USE_NEW_INT */
   if (mice->intdrv) 
     mouse_int();
   else 
@@ -1398,7 +1709,7 @@ static void inte6(u_char i) {
 /* mfs FCB call */
 static void inte7(u_char i) {
   SETIVEC(0xe7, INTE7_SEG, INTE7_OFF);
-  run_int(0xe7);
+  real_run_int(0xe7);
 }
 
 #ifdef USE_INT_QUEUE
@@ -1445,29 +1756,18 @@ static void inte8(u_char i) {
  * DANG_END_FUNCTION
  */
 
-void
-do_int(int i)
+void do_int(int i)
 {
+#ifndef USE_NEW_INT
   void (* caller_function)();
 
-#if 0 /* 94/07/02 Just for reference */
-  saytime("do_int");
-#endif
-#if 0 /* 94/07/02 Just for reference */
-if (i== 0x2f) {
-  show_regs(__FILE__, __LINE__);
-  k_printf("Do INT0x%02x, eax=0x%04x, ebx=0x%04x ss=0x%04x esp=0x%04x cs=0x%04x ip=0x%04x CF=%d \n", i, LWORD(eax), LWORD(ebx), LWORD(ss), LWORD(esp), LWORD(cs), LWORD(eip), (int)REG(eflags) & CF);
-}
-#endif
-
   if ((LWORD(cs) != BIOSSEG) && IS_REDIRECTED(i) && can_revector(i)){
-    run_int(i);
+    real_run_int(i);
     return;
   }
 
   caller_function = interrupt_function[i];
   caller_function(i);
-#if 1 
   /* This is a kludge to avoid immediate respawning of dosemu
    * if we have cs:ip pointing to the BIOS stubs' IRET
    * We do the IRET ourselves, so we directly let the DOSapp
@@ -1491,9 +1791,59 @@ if (i== 0x2f) {
       else  WRITE_FLAGSE((READ_FLAGSE() | IF) & ~VIF);
     }
   }
+#else /* USE_NEW_INT */
+ 	unsigned long magic_address;
+	if (in_dpmi) {
+		if (dpmi_eflags & IF) {
+			set_IF();
+		} else {
+			clear_IF();
+		}
+	}
+	
+ 	if ((d.defint > 2) && (((i != 0x28) && (i != 0x2f)) || in_dpmi)) {
+ 		di_printf("Do INT0x%02x eax=0x%08x ebx=0x%08x ss=0x%08x esp=0x%08x\n"
+ 			  "           ecx=0x%08x edx=0x%08x ds=0x%08x  cs=0x%08x ip=0x%08x\n"
+ 			  "           esi=0x%08x edi=0x%08x es=0x%08x flg=0x%08x\n",
+ 			  i, _EAX, _EBX, _SS, _ESP,
+ 			  _ECX, _EDX, _DS, _CS, _IP,
+ 			  _ESI, _EDI, _ES, (int) read_EFLAGS());
+ 	}
+	
+#if 1  /* This test really ought to be in the main loop before
+ 	*  instruction execution not here. --EB 10 March 1997 
+ 	*/
+	
+ 	/* try to catch jumps to 0:0 (e.g. uninitialized user interrupt vectors),
+ 	   which sometimes can crash the whole system, not only dosemu... */
+ 	if (SEGOFF2LINEAR(_CS, _IP) < 1024) {
+ 		dbug_printf("OUCH! attempt to execute interrupt table - quickly dying\n");
+ 		leavedos(57);
+ 	}
 #endif
+  
+  
+ 	/* see if I want to use the caller function */
+ 	/* I want to use it if I must always use it, or I am calling into the
+ 	   interrupt table at the start of the dosemu bios */
+ 	/* assume IP was just incremented by 2 past int int instruction which set us
+ 	   off */
+	
+ 	magic_address = SEGOFF2LINEAR(BIOSSEG, INT_OFF(i));
+ 	if (magic_address == (SEGOFF2LINEAR(_CS, _IP) -2)) {
+ 		run_caller_func(i, FALSE);
+ 	}
+ 	else if ((magic_address == IVEC(i)) ||
+ 		 (can_revector(i) == REVECT)) {
+ 		run_caller_func(i, TRUE);
+ 	}
+ 	else {
+ 		real_run_int(i);
+ 	}
+#endif /* USE_NEW_INT */
 }
 
+
 #ifdef USE_INT_QUEUE
 /*
  * Called to queue a hardware interrupt - will call "callstart"
@@ -1567,13 +1917,6 @@ void int_queue_run(void)
   ssp = (unsigned char *) (REG(ss) << 4);
   sp = (unsigned long) LWORD(esp);
 
-#ifndef NEW_KBD_CODE
-  if (current_interrupt == 0x09) {
-    k_printf("Int9 set\n");
-    /* If another program does a keyboard read on port 0x60, we'll know */
-    read_next_scancode_from_queue();
-  }
-#endif
    
   /* call user startup function...don't run interrupt if returns -1 */
   if (int_queue[int_queue_start].callstart) {
@@ -1654,21 +1997,21 @@ void setup_interrupts(void) {
 
   /* init trapped interrupts called via jump */
   for (i = 0; i < 256; i++) {
-#if 0 /* Rick's BUG FIXER */
-    /* Go figure? */
-    if (i == 115)
-      continue;
-#endif
     interrupt_function[i] = default_interrupt;
     if ((i & 0xf8) == 0x60) { /* user interrupts */
 	/* show also EMS (int0x67) as disabled */
 	SETIVEC(i, 0, 0);
     } else {
+#ifndef USE_NEW_INT
 	SETIVEC(i, BIOSSEG, 16 * i);
+#else /* USE_NEW_INT */
+	SETIVEC(i, BIOSSEG, INT_OFF(i));
+#endif /* USE_NEW_INT */
     }
   }
   
   interrupt_function[5] = int05;
+#ifndef USE_NEW_INT
   interrupt_function[8] = int08;
   interrupt_function[9] = int09;
   interrupt_function[0xa] = int_a_b_c_d_e_f;
@@ -1677,6 +2020,7 @@ void setup_interrupts(void) {
   interrupt_function[0xd] = int_a_b_c_d_e_f;
   interrupt_function[0xe] = int_a_b_c_d_e_f;
   interrupt_function[0xf] = int_a_b_c_d_e_f;
+#endif /* not USE_NEW_INT */
   /* This is called only when revectoring int10 */
   interrupt_function[0x10] = int10;
   interrupt_function[0x11] = int11;
@@ -1684,7 +2028,9 @@ void setup_interrupts(void) {
   interrupt_function[0x13] = int13;
   interrupt_function[0x14] = int14;
   interrupt_function[0x15] = int15;
+#ifndef USE_NEW_INT
   interrupt_function[0x16] = int16;
+#endif /* not USE_NEW_INT */
   interrupt_function[0x17] = int17;
   interrupt_function[0x18] = int18;
   interrupt_function[0x19] = int19;
@@ -1707,7 +2053,13 @@ void setup_interrupts(void) {
 #endif
 
   /* Let kernel handle this, no need to return to DOSEMU */
+#ifndef USE_NEW_INT
   SETIVEC(0x1c, 0xf010, 0xc0);
+#else /* USE_NEW_INT */
+ #if 0
+  SETIVEC(0x1c, BIOSSEG + 0x10, INT_OFF(0x1c) +2 - 0x100);
+ #endif
+#endif /* USE_NEW_INT */
 
   /* show EMS as disabled */
   SETIVEC(0x67, 0, 0);
@@ -1719,15 +2071,26 @@ void setup_interrupts(void) {
     seg = (u_short *) (Mouse_ADD + 10);
     /* tell the mouse driver where we are...exec add, seg, offset */
     mouse_sethandler(ptr, seg, off);
+#ifndef USE_NEW_INT
     SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
     SETIVEC(0x33, Mouse_SEG, Mouse_ROUTINE_OFF);
   }
   else
     *(unsigned char *) (BIOSSEG * 16 + 16 * 0x33) = 0xcf;	/* IRET */
+#else /* USE_NEW_INT */
+  }
+  else {
+    /* point to the retf#2 immediately after the int 33 entry point. */
+    SETIVEC(0x33, BIOSSEG, INT_OFF(0x33)+2);
+  }
+#endif /* USE_NEW_INT */
 
   SETIVEC(0x16, INT16_SEG, INT16_OFF);
   SETIVEC(0x09, INT09_SEG, INT09_OFF);
   SETIVEC(0x08, INT08_SEG, INT08_OFF);
+#ifdef NEW_CMOS
+  SETIVEC(0x70, INT70_SEG, INT70_OFF);
+#endif
 
   /* Install new handler for video-interrupt into bios_f000_int10ptr,
    * for video initialization at f800:4200
@@ -1744,9 +2107,26 @@ void setup_interrupts(void) {
   if (config.dualmon == 2) {
     interrupt_function[0x42] = interrupt_function[0x10];
   }
+#ifndef USE_NEW_INT
   else *(u_char *) 0xff065 = 0xcf;	/* IRET */
+#endif /* not USE_NEW_INT */
 }
 
+static struct revectored_struct saved_int21_revectored = {{0,0,0,0,0,0,0,0}};
+
+void set_int21_revectored(int all)
+{
+  static int last_all = 0;
+  if (all < 0)
+    all = last_all;
+  if (all > 0)
+    memset(&vm86s.int21_revectored, 0xff, sizeof(vm86s.int21_revectored));
+  else
+    vm86s.int21_revectored = saved_int21_revectored;
+  last_all = all;
+}
+
+
 /*
  * DANG_BEGIN_FUNCTION int_vector_setup
  *
@@ -1773,7 +2153,10 @@ void int_vector_setup(void)
   for (i=0; i<0x100; i++)
     if (can_revector_int21(i)==REVECT)
       set_revectored(i, &vm86s.int21_revectored);
+  saved_int21_revectored = vm86s.int21_revectored;
+  set_int21_revectored(-1);
 #endif
+
 #ifdef __NetBSD__
   memset(&vm86s.int_byuser[0], 0x00, sizeof(vm86s.int_byuser));
   memset(&vm86s.int21_byuser[0], 0x00, sizeof(vm86s.int21_byuser));
