@@ -1,13 +1,40 @@
-/* this handles the int10 video functions.
-   Most functions here change only the video memory and status 
-   variables; the actual screen is then rendered asynchronously 
-   after these by Video->update_screen.
+/* 
+ * DANG_BEGIN_MODULE
+ *
+ * REMARK
+ * Video BIOS implementation.
+ *
+ * This module handles the int10 video functions.
+ * Most functions here change only the video memory and status 
+ * variables; the actual screen is then rendered asynchronously 
+ * after these by Video->update_screen.
+ *
+ * /REMARK
+ * DANG_END_MODULE
+ *
+ * DANG_BEGIN_CHANGELOG
+ *
+ * 5/24/95, Erik Mouw (J.A.K.Mouw@et.tudelft.nl) and 
+ * Arjan Filius (I.A.Filius@et.tudelft.nl)
+ * changed int10() to make graphics work with X.
+ *
+ * 1998/04/05: Put some work into set_video_mode() (made it
+ * more VGA comaptible) and removed new_set_video_mode().
+ * Removed (useless) global variable "gfx_mode".
+ * -- sw (Steffen.Winterfeldt@itp.uni-leipzig.de)
+ * 
+ * Readded new_set_video_mode, its needed for non-X compiles.
+ * -- EB 3 June 1998
+ *
+ * Renamed new_set_video_mode to X_set_video_mode to avoid confusion
+ * in the future
+ * -- Hans 980614
+ *
+ * DANG_END_CHANGELOG
+ */
 
-  5/24/95, Erik Mouw (J.A.K.Mouw@et.tudelft.nl) and 
-  Arjan Filius (I.A.Filius@et.tudelft.nl)
-  changed int10() to make graphics work with X.
-*/
 
+#include <string.h>
 #include "config.h"
 
 #include "emu.h"
@@ -56,6 +83,9 @@
 inline void set_cursor_shape(ushort shape) {
    int cs,ce;
 
+   if (config.cardtype == CARD_NONE)
+     return;
+
    cs=CURSOR_START(shape) & 0x1F;
    ce=CURSOR_END(shape) & 0x1F;
 
@@ -99,6 +129,9 @@ Scroll(us *sadr, int x0, int y0, int x1, int y1, int l, int att)
   us blank = ' ' | (att << 8);
   us tbuf[MAX_COLUMNS];
 
+  if (config.cardtype == CARD_NONE)
+     return;
+
   if ( (config.cardtype == CARD_MDA) && ((att & 7) != 0) && ((att & 7) != 7) )
     {
       blank = ' ' | ((att | 7) << 8);
@@ -187,6 +220,9 @@ void bios_scroll(int x0,int y0,int x1,int y1,int n,byte attr) {
    struct scroll_entry *s;
    int sqh2;
 
+   if (config.cardtype == CARD_NONE)
+     return;
+
    VIDEO_UPDATE_LOCK();
    sqh2=SQ_INC(sq_head);
    if (n!=0 && !Video->is_mapped && sqh2!=sq_tail) {
@@ -226,9 +262,16 @@ char_out(unsigned char ch, int s)
 {
   int newline_att = 7;
   int xpos, ypos;
+
 #if USE_DUALMON
   int virt_text_base = BIOS_SCREEN_BASE;
 #endif
+
+  if (config.cardtype == CARD_NONE) {
+     putchar (ch);
+     return;
+  }
+
   xpos = get_bios_cursor_x_position(s);
   ypos = get_bios_cursor_y_position(s);
 
@@ -299,6 +342,9 @@ clear_screen(int s, int att)
   int virt_text_base = BIOS_SCREEN_BASE;
 #endif
 
+  if (config.cardtype == CARD_NONE)
+     return;
+
   v_printf("VID: cleared screen: %d %d %p\n", s, att, SCREEN_ADR(s));
   if (s > max_page) return;
   
@@ -315,6 +361,145 @@ clear_screen(int s, int att)
 }
 
 
+#if X_GRAPHICS
+
+/*
+ * set_video_mode() accepts both (S)VGA and VESA mode numbers
+ * It should be fully compatible with the old set_video_mode()
+ * function.
+ * Note: bit 16 in "mode" is used internally to indicate that
+ * nothing should be done except adjusting the font size.
+ * -- 1998/04/04 sw
+ */
+
+static boolean X_set_video_mode(int mode) {
+  vga_mode_info *vmi;
+  int clear_mem = 1;
+  int adjust_font_size = 0;
+  
+  if (config.cardtype == CARD_NONE) {
+    v_printf("set_video_mode: no video!\n");
+    return 0;
+  }
+
+
+  if(mode & (1 << 16)) {
+    adjust_font_size = 1;
+    mode &= ~(1 << 16);
+  }
+
+  mode &= 0xffff;
+
+  if(mode != video_mode) adjust_font_size = 0;
+
+  v_printf("set_video_mode: mode = 0x%02x%s\n", mode, adjust_font_size ? " (adjust font size)" : "");
+
+  if((vmi = vga_emu_find_mode(mode, NULL)) == NULL) {
+    v_printf("set_video_mode: undefined video mode\n");
+    return 0;
+  }
+
+  if(Video->setmode == NULL) {
+    v_printf("set_video_mode: no setmode handler!\n");
+    return 0;
+  }
+
+  if(adjust_font_size) {
+    text_scanlines = vmi->height;
+    li = text_scanlines / vga_font_height;
+    if(li > MAX_LINES) li = MAX_LINES;
+    WRITE_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1, li - 1);
+    WRITE_WORD(BIOS_FONT_HEIGHT, vga_font_height);
+    Video->setmode(vmi->mode_class, co, li);
+    return 1;
+  }
+
+  video_mode = mode;
+
+  if(mode >= 0x80 && mode < 0x100) {
+    mode &= 0x7f;
+    clear_mem = 0;
+  }
+  if(mode & 0x8000) {
+    mode &= ~0x8000;
+    clear_mem = 0;
+  }
+
+  if(config.cardtype == CARD_MDA) video_mode = 7;
+
+  /*
+   * We store the SVGA mode number (if possible) even when setting a VESA mode. -- sw
+   */
+  WRITE_BYTE(BIOS_VIDEO_MODE, vmi->mode & 0x7f);
+  WRITE_BYTE(BIOS_CURRENT_SCREEN_PAGE, 0);
+
+  if(vmi->mode_class == TEXT) {
+    clear_scroll_queue();
+  }
+
+  li = vmi->text_height;
+  co = vmi->text_width;
+  if(li > MAX_LINES) li = MAX_LINES;
+  vga_font_height = vmi->char_height;
+  text_scanlines = vmi->height;
+
+  WRITE_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1, li - 1);
+  WRITE_WORD(BIOS_SCREEN_COLUMNS, co);
+  WRITE_WORD(BIOS_FONT_HEIGHT, vga_font_height);
+
+#if USE_DUALMON
+  /*
+   * The following code (& comment) is taken literally
+   * from the old set_video_mode(). Don't know what it's
+   * for (or if it works). -- 1998/04/04 sw 
+   */
+
+  /*
+   * This to be sure in case of older DOS programs probing HGC.
+   * There was no secure way to detect a HGC before VGA was invented.
+   * ( Now we can do INT 10, AX=1A00 ).
+   * Some older DOS programs do it by modifying EQUIP-flags
+   * and then let the BIOS say, if it can ?!?!)
+   * If we have config.dualmon, this happens legaly.
+   */
+  if(config.dualmon)
+    Video->setmode(7, co, li);
+  else
+#endif
+
+  /* setmode needs video_mode to _still have_ the memory-clear bit -- sw */
+  Video->setmode(vmi->mode_class, co, li);
+
+  /*
+   * video_mode is expected to be the mode number _without_ the
+   * memory-clear bit
+   * -- sw
+   */
+  video_mode = mode;
+
+  if(clear_mem && vmi->mode_class == TEXT) clear_screen(0, 7);
+
+  WRITE_BYTE(BIOS_VIDEO_INFO_0, clear_mem ? 0x60 : 0xe0);
+  memset((void *) 0x450, 0, 0x10);	/* equiv. to set_bios_cursor_(x/y)_position(0..7, 0) */
+
+  video_page = 0;
+  WRITE_WORD(BIOS_VIDEO_MEMORY_ADDRESS, 0);
+  screen_adr = SCREEN_ADR(0);
+  screen_mask = 0;
+  cursor_col = get_bios_cursor_x_position(0);
+  cursor_row = get_bios_cursor_y_position(0);
+
+  /*
+   * There are still some BIOS variables that need to be updated.
+   * (cursor shape, CRTC address...)
+   * -- sw
+   */
+
+  return 1;
+}    
+#endif /* X_GRAPHICS */
+
+
 /* XXX- shouldn't this reset the current video page to 0 ? 
 */
 
@@ -324,6 +509,9 @@ boolean set_video_mode(int mode) {
 
   v_printf("set_video_mode: mode = 0x%02x\n",mode);
   
+#ifdef X_GRAPHICS
+  if (Video == &Video_X) return X_set_video_mode(mode);
+#endif  
   oldco = co;
   old_video_mode = video_mode;
   video_mode=mode&0x7f;
@@ -386,7 +574,6 @@ boolean set_video_mode(int mode) {
 #endif
     
 do_text_mode:
-    gfx_mode = TEXT;	        /* we're in a text mode now */
     clear_scroll_queue();
 
     li=text_scanlines/vga_font_height;
@@ -402,15 +589,9 @@ do_text_mode:
     if ( config.cardtype == CARD_MDA )
       mode = 7;
        
-    WRITE_BYTE(BIOS_VIDEO_MODE, video_mode=mode&0x7f);
     break;
 
-#ifdef NEW_X_CODE
-case 0x0d:
-case 0x0e:
-case 0x10:
-case 0x12:
-#endif
+
 case 0x13:	/*Not finished ! */
 case 0x5c:
 case 0x5d:
@@ -426,6 +607,9 @@ case 0x62:
     v_printf("undefined video mode 0x%x\n", mode);
   goto error;
   }
+
+  WRITE_BYTE(BIOS_VIDEO_MODE, video_mode=mode&0x7f);
+
   if (oldco != co)
     WRITE_WORD(BIOS_SCREEN_COLUMNS, co);
   return 1;
@@ -436,64 +620,6 @@ error:
   return 0;
 }    
 
-#ifdef NEW_X_CODE
-/*
- * new_set_video_mode() is called to set a VESA mode as set_video_mode()
- * really has problems getting its job done.
- */
-
-boolean new_set_video_mode(int mode) {
-  vga_mode_info *vmi;
-  int clear_mem = 1;
-
-  mode &= 0xffff;
-
-  v_printf("new_set_video_mode: mode = 0x%02x\n", mode);
-
-  if((vmi = vga_emu_find_mode(mode, NULL)) == NULL) {
-    v_printf("new_set_video_mode: undefined video mode\n");
-    return 0;
-  }
-
-  if(Video->setmode == NULL) {
-    v_printf("new_set_video_mode: no setmode handler!\n");
-    return 0;
-  }
-
-  video_mode = mode;
-
-  if(mode >= 0x80 && mode < 0x100) {
-    mode &= 0x7f;
-    clear_mem = 0;
-  }
-  if(mode & 0x8000) {
-    mode &= ~0x8000;
-    clear_mem = 0;
-  }
-
-  if(config.cardtype == CARD_MDA) video_mode = 7;
-
-  WRITE_BYTE(BIOS_VIDEO_MODE, video_mode);
-
-  if(vmi->mode_class == TEXT) {
-    gfx_mode = TEXT;
-    clear_scroll_queue();
-    if(clear_mem) clear_screen(READ_BYTE(BIOS_CURRENT_SCREEN_PAGE), 7);
-  }
-
-  li = vmi->text_height;
-  co = vmi->text_width;
-  if(li > MAX_LINES) li = MAX_LINES;
-  WRITE_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1, li - 1);
-  WRITE_WORD(BIOS_SCREEN_COLUMNS, co);
-  WRITE_WORD(BIOS_FONT_HEIGHT, vmi->char_height);
-  Video->setmode(vmi->mode_class, co, li);
-
-  if(clear_mem && vmi->mode_class == TEXT) clear_screen(READ_BYTE(BIOS_CURRENT_SCREEN_PAGE), 7);
-
-  return 1;
-}    
-#endif
 
 /******************************************************************/
 
@@ -560,6 +686,7 @@ void int10()
     page = HI(bx);
     x = LO(dx);
     y = HI(dx);
+    v_printf("set cursor: pg:%d x:%d y:%d\n", page, x, y);
     if (page > 7) {
       v_printf("ERROR: video error (setcur/page>7: %d)\n", page);
       CARRY;
@@ -600,11 +727,7 @@ void int10()
       }
       if (config.console_video) set_vc_screen_page(page);
 #if X_GRAPHICS
-#ifdef NEW_X_CODE
       if (config.X) vga_emu_set_text_page(page, TEXT_SIZE);
-#else
-      if (config.X) set_vgaemu_page(page);      /*root@sjoerd*/
-#endif
 #endif
 
       WRITE_BYTE(BIOS_CURRENT_SCREEN_PAGE, video_page = page);
@@ -694,9 +817,47 @@ void int10()
       
       if (LO(ax) < 2) {                  /* use attribute in BL */
         attr=LO(bx)<<8;
+#if 1					/* 1998/03/13 sengoku@intsys.co.jp */
+        x += n;
+	while(n--) {
+	  /* *(sadr++) = *src++ | attr; */
+          if (*src <= 0x1a) {
+            switch(*src) {
+              case 0x0d :
+                sadr = (u_short *) SCREEN_ADR(page) + y*co, x = n;
+                src++;
+                break;
+              case 0x0a :
+                sadr += co, x--, y++;
+                src++;
+                break;
+              case 0x07 : /* beep */
+#if 0  /* ignored, we have to check wether it breaks the speaker stuff --Hans */
+                putchar(0x07);
+#else
+                /* this should do what we want, says Eric 980425 */
+                speaker_on(125, 0x637);
+#endif
+                x--;
+                src++;
+                break;
+              case 0x08 : /* backspace */
+                sadr--, x--;
+                src++;
+                break;
+              default:
+                WRITE_WORD(sadr++, READ_BYTE(src++) | attr);
+                break;
+            }
+          } else {
+            WRITE_WORD(sadr++, READ_BYTE(src++) | attr);
+          }
+        }
+#else
 	while(n--)
 	  /* *(sadr++) = *src++ | attr; */
-	  WRITE_WORD(sadr++, READ_BYTE(src++) | attr);
+            WRITE_WORD(sadr++, READ_BYTE(src++) | attr);
+#endif
       }
       else {                             /* use attributes in buffer */
 	/*memcpy(sadr,src,n*2);*/
@@ -866,7 +1027,7 @@ void int10()
 	
       more_lines:
 	{
-	  if (!set_video_mode(video_mode | 0x80)) 
+	  if(!set_video_mode(video_mode | (1 << 16))) 
 	    {
 	      li = old_li; /* not that it changed */
 	      vga_font_height = old_font_height;
