@@ -7,6 +7,7 @@ static char *id= "$Id: data.c,v 1.4 1995/04/08 22:29:37 root Exp $";
 
 #include <termios.h>
 #include <sys/types.h>
+#include "config.h"
 #include "emu.h"
 #include "xms.h"
 #include "disks.h"
@@ -30,7 +31,6 @@ static char *id= "$Id: data.c,v 1.4 1995/04/08 22:29:37 root Exp $";
 #include "dma.h"
 #include "sound.h"
 #include "serial.h"
-#ifdef NEW_KBD_CODE
+#include "dosemu_debug.h"
 #include "keyb_server.h"
 #include "keyb_clients.h"
-#endif
