@@ -47,6 +47,8 @@
 
 #include <unistd.h>
 
+/* #define EXCESSIVE_DEBUG 1 */
+
 typedef struct {
   Bit8u lsb;
   Bit8u msb;
@@ -60,6 +62,7 @@ typedef union {
 } multi_t;
 
 typedef struct {
+  int     mask;          /* Whether channel is masked */
   int     wfd;           /* Write part of pipe */
   int     rfd;           /* Read part of pipe */
   int     dack;          /* DACK */
@@ -127,6 +130,8 @@ inline int get_mask (int ch);
 size_t dma_do_read(int controller, int channel, Bit32u target_addr);
 size_t dma_do_write(int controller, int channel, Bit32u target_addr);
 
+static inline void activate_channel (int controller, int channel);
+static inline void deactivate_channel (int controller, int channel);
 
 /* PUBLIC CODE */
 
@@ -155,8 +160,10 @@ void dma_assert_DREQ(int channel)
 
   controller = channel & 4;
   ch = channel & DMA_CH_SELECT;
-
   mask = 1 << (ch + 4);
+  
+  if(!dma[controller].i[ch].mask)
+    activate_channel (controller, ch);
 
   dma[controller].status |= mask;
 }
@@ -187,6 +194,8 @@ void dma_drop_DREQ(int channel)
   controller = channel & 4;
   ch = channel & DMA_CH_SELECT;
 
+  deactivate_channel(controller, ch);
+
   mask = 1 << (ch +4);
   dma[controller].status &= ~ mask;
 }
@@ -356,6 +365,41 @@ inline int has_underflow (multi_t data)
   return data.bits.underflow;
 }
 
+/*
+ * This ensures that the controller will be called to service this channel.
+ */
+static inline void activate_channel (int controller, int channel)
+{
+  int mask, ch;
+
+  ch = get_ch (controller, channel);
+
+  dma[controller].i[channel].mask = 0;
+  /* 
+   * We only _actually_ activate the channel if DREQ is set. It's irrelevant
+   * otherwise (from an idea by Michael Karcher) - AM
+   */
+     
+  if (dma_test_DREQ(ch) ) {
+    mask = get_mask (ch);
+
+    is_dma |= mask;
+  }
+}
+
+/* 
+ * This ensures that the controller is not called for this channel.
+ */
+static inline void deactivate_channel (int controller, int channel)
+{
+  int mask, ch;
+
+  dma[controller].i[channel].mask = 1;
+  ch = get_ch (controller, channel);
+  mask = get_mask (ch);
+
+  is_dma &= ~mask;  
+}
 
 inline Bit32u create_addr(Bit8u page, multi_t address, int dma_c)
 {
@@ -430,6 +474,11 @@ void dma_recover_values (int controller, int channel)
   dma[controller].i[channel].size = dma[controller].i[channel].set_size;
 }
 
+/* 
+ * Obselete? 
+ * Might make a useful public function if converted to just take a single
+ * parameter - AM 
+ */
 Bit16u length_transferred (int controller, int channel)
 {
   return (get_value (dma[controller].i[channel].length)
@@ -514,6 +563,8 @@ Bit8u dma_io_read(ioport_t port)
 inline void dma_write_addr (int dma_c, int channel, Bit8u value)
 {
   dma[dma_c].address[channel].data[dma[dma_c].ff] = value;
+  dma[dma_c].i[channel].address.data[dma[dma_c].ff] = value; /* autoinit */
+
   h_printf ("DMA: Wrote 0x%x into Channel %d Address (Byte %d)\n", value,
 	    (dma_c *4) + channel, dma[dma_c].ff);
 
@@ -523,6 +574,8 @@ inline void dma_write_addr (int dma_c, int channel, Bit8u value)
 inline void dma_write_count (int dma_c, int channel, Bit8u value)
 {
   dma[dma_c].length[channel].data[dma[dma_c].ff] = value;
+  dma[dma_c].i[channel].length.data[dma[dma_c].ff] = value; /* autoinit */
+
   h_printf ("DMA: Wrote 0x%x into Channel %d Length (Byte %d)\n", value,
 	    (dma_c *4) + channel, dma[dma_c].ff);
 
@@ -534,16 +587,18 @@ inline void dma_write_mask (int dma_c, Bit8u value)
     if (value & DMA_SELECT_BIT)
     {
       dma[dma_c].current_channel = (int) value & 3;
+
+      deactivate_channel (dma_c, dma[dma_c].current_channel); /* - Karcher */
+
       h_printf ("DMA: Channel %u selected.\n", 
 			(dma_c * 4) + dma[dma_c].current_channel);
     }
     else
     {
-      /* I don't "trust" 'value' so I use our recorded value - AM */
       h_printf ("DMA: Channel %u deselected.\n", 
-			 (dma_c * 4) + dma[dma_c].current_channel);
+			 (dma_c * 4) + (value&3));
 
-      is_dma |= (1 << dma[dma_c].current_channel);
+      activate_channel (dma_c, dma[dma_c].current_channel);
 
       dma[dma_c].current_channel = -1;
     }
@@ -711,31 +766,22 @@ void dma_install_handler (int ch, int wfd, int rfd,
 
 
 /*
- * This sets up a DMA channel for the first time.
+ * This sets up a DMA channel when it used for the first time after 
+ * programming ie if there are no files alreadu associated with it.
  */
 
 int dma_initialise_channel (int controller, int channel) 
 {
 	extern dma_t dma[2];
 	int tmp_pipe[2];
-	int ch, mask;
+	int ch/*, mask*/;
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch); */
 
 	h_printf ("DMA: Initialising transfer for channel %d, controller %d\n",
 		  ch, controller +1);
 
-	/* Backup the values into the private store - in case we need them */
-	dma[controller].i[channel].address.data[0] 
-	  = dma[controller].address[channel].data[0];
-	dma[controller].i[channel].address.data[1] 
-	  = dma[controller].address[channel].data[1];
-	dma[controller].i[channel].length.data[0] 
-	  = dma[controller].length[channel].data[0];
-	dma[controller].i[channel].length.data[1] 
-	  = dma[controller].length[channel].data[1];
-
 	if (pipe(tmp_pipe)) {
 	  /* Failed to create the pipe */
 	  h_printf ("DMA: Failed to intialise transfer for channel %d on controller %d\n", 
@@ -745,7 +791,9 @@ int dma_initialise_channel (int controller, int channel)
 		 * DANG_FIXTHIS: Pipe Creation failed. Lets hope that it times out ! 
 		 */
 
-		is_dma &= ~mask;
+	  /*is_dma &= ~mask; */
+	        deactivate_channel (controller, channel);
+
 		return -1;
 	}
 
@@ -810,10 +858,10 @@ void dma_process_demand_mode_write (int controller, int channel)
 	Bit32u target_addr;
 	int amount_done = 0;
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
       target_addr = create_addr (dma[controller].page[channel],
 				  dma[controller].address[channel],
@@ -838,8 +886,9 @@ void dma_process_demand_mode_write (int controller, int channel)
 	  } else {
 	    dma[controller].status |= (1 << channel);
 
-	    is_dma &= ~ (1 << (ch -1));
-	    
+	    /*is_dma &= ~ (1 << (ch -1));*/
+	    deactivate_channel (controller, channel);
+
 	    if (dma[controller].i[channel].handler != NULL) {
 	      /* The handler is expected to close any descriptors */
 	      dma[controller].i[channel].handler (DMA_HANDLER_DONE, 0);
@@ -860,7 +909,8 @@ void dma_process_demand_mode_write (int controller, int channel)
 	    dma[controller].i[channel].wfd = -1;
 	    */ 
 		
-		is_dma &= ~mask;
+	    deactivate_channel (controller, channel);
+		/*		is_dma &= ~mask;*/
 	    return;
 	  }
 
@@ -886,7 +936,7 @@ void dma_process_demand_mode_write (int controller, int channel)
 
 		if (dma[controller].i[channel].handler != NULL) {
 	      dma[controller].i[channel].handler (DMA_HANDLER_WRITE,
-				 length_transferred (controller, channel));
+						  amount_done);
 	  }
 	}
 }
@@ -896,10 +946,10 @@ void dma_process_single_mode_write (int controller, int channel)
 	Bit32u target_addr;
 	int amount_done = 0;
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
 	target_addr = create_addr(dma[controller].page[channel],
 				  dma[controller].address[channel],
@@ -910,7 +960,8 @@ void dma_process_single_mode_write (int controller, int channel)
 	    /* Transfer is complete */
 	    dma[controller].status |= (1 << channel);
 
-		is_dma &= ~mask;
+	        deactivate_channel (controller, channel);
+		/*		is_dma &= ~mask;*/
 				
 		if (dma[controller].i[channel].handler != NULL) {
 			/* The handler is expected to close any descriptors */
@@ -952,7 +1003,7 @@ void dma_process_single_mode_write (int controller, int channel)
 
 		if (dma[controller].i[channel].handler != NULL) {
 	      dma[controller].i[channel].handler (DMA_HANDLER_WRITE,
-				 length_transferred (controller, channel));
+						  amount_done);
 	  }
 	}
 }	
@@ -963,10 +1014,10 @@ void dma_process_block_mode_write (int controller, int channel)
 	Bit32u target_addr;
 	int amount_done = 0;
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
 	target_addr = create_addr(dma[controller].page[channel],
 				  dma[controller].address[channel],
@@ -991,7 +1042,8 @@ void dma_process_block_mode_write (int controller, int channel)
 	  } else {
 	    dma[controller].status |= (1 << channel);
 
-	    is_dma &= ~mask;
+	    deactivate_channel (controller, channel);
+		/*	    is_dma &= ~mask;*/
 		
 	    if (dma[controller].i[channel].handler != NULL) {
 	      /* The handler is expected to close any descriptors */
@@ -1013,7 +1065,8 @@ void dma_process_block_mode_write (int controller, int channel)
 /*	    close (dma[controller].i[channel].wfd);
 	    dma[controller].i[channel].wfd = -1;
 	    */ 
-		is_dma &= ~mask;
+	        deactivate_channel (controller, channel);
+		/*		is_dma &= ~mask;*/
 	    return;
 	  }
 
@@ -1041,7 +1094,7 @@ void dma_process_block_mode_write (int controller, int channel)
 	    
 		if (dma[controller].i[channel].handler != NULL) {
 	      dma[controller].i[channel].handler (DMA_HANDLER_WRITE,
-				 length_transferred (controller, channel));
+						  amount_done);
 	  }
 	}
 }
@@ -1053,10 +1106,10 @@ void dma_process_block_mode_write (int controller, int channel)
 void dma_process_cascade_mode_write (int controller, int channel)
 {
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
 	  h_printf ("DMA: Attempt to use unsupported CASCADE mode\n");
 
@@ -1064,7 +1117,8 @@ void dma_process_cascade_mode_write (int controller, int channel)
 	  dma[controller].i[channel].wfd = -1;
 	  */ 
 
-	is_dma &= ~mask;
+	  deactivate_channel (controller, channel);
+	  /*	is_dma &= ~mask;*/
 
 	if (dma[controller].i[channel].handler != NULL) {
 		dma[controller].i[channel].handler(DMA_HANDLER_DONE, 0);
@@ -1077,10 +1131,10 @@ void dma_process_demand_mode_read (int controller, int channel)
 	Bit32u target_addr;
 	int amount_done = 0;
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
 	target_addr = create_addr(dma[controller].page[channel],
 				  dma[controller].address[channel],
@@ -1105,7 +1159,8 @@ void dma_process_demand_mode_read (int controller, int channel)
 	  } else {
 	    dma[controller].status |= (1 << channel);
 
-	    is_dma &= ~mask;
+	    deactivate_channel (controller, channel);
+	    /*is_dma &= ~mask;*/
 	    
 	    if (dma[controller].i[channel].handler != NULL) {
 	      /* The handler is expected to close any descriptors */
@@ -1126,7 +1181,8 @@ void dma_process_demand_mode_read (int controller, int channel)
 /*	    close (dma[controller].i[channel].rfd);
 	    dma[controller].i[channel].rfd = -1;
 	    */ 
-		is_dma &= ~mask;
+	    deactivate_channel (controller, channel);
+	    /*is_dma &= ~mask;*/
 	    return;
 	  }
 
@@ -1154,7 +1210,7 @@ void dma_process_demand_mode_read (int controller, int channel)
 
 		if (dma[controller].i[channel].handler != NULL) {
 	      dma[controller].i[channel].handler (DMA_HANDLER_READ,
-				 length_transferred (controller, channel));
+						  amount_done);
 		}
 	}
 }
@@ -1165,10 +1221,10 @@ void dma_process_single_mode_read (int controller, int channel)
 	Bit32u target_addr;
 	size_t amount_done = 0;
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
 	target_addr = create_addr(dma[controller].page[channel],
 				  dma[controller].address[channel],
@@ -1205,7 +1261,8 @@ void dma_process_single_mode_read (int controller, int channel)
 
 	    dma[controller].status |= (1 << channel);
 
-	    is_dma &= ~mask;
+	    deactivate_channel (controller, channel);
+	    /*is_dma &= ~mask;*/
 				
 	    dma[controller].i[channel].rfd = -1;
 	  }		
@@ -1249,7 +1306,7 @@ void dma_process_single_mode_read (int controller, int channel)
 
 		if (dma[controller].i[channel].handler != NULL) {
 	      dma[controller].i[channel].handler (DMA_HANDLER_READ,
-				 length_transferred (controller, channel));
+						  amount_done);
 	  }
 	}
 	h_printf("DMA: [crisk] DMA single read end trace\n");
@@ -1262,10 +1319,10 @@ void dma_process_block_mode_read (int controller, int channel)
 	Bit32u target_addr;
 	int amount_done = 0;
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
 	target_addr = create_addr(dma[controller].page[channel],
 				  dma[controller].address[channel],
@@ -1290,7 +1347,8 @@ void dma_process_block_mode_read (int controller, int channel)
 	  } else {
 	    dma[controller].status |= (1 << channel);
 
-	    is_dma &= ~mask;
+	    deactivate_channel (controller, channel);
+	    /*is_dma &= ~mask;*/
 
 	    if (dma[controller].i[channel].handler != NULL) {
 	      /* The handler is expected to close any descriptors */
@@ -1312,7 +1370,8 @@ void dma_process_block_mode_read (int controller, int channel)
 	    dma[controller].i[channel].rfd = -1;
 	    */ 
 
-		is_dma &= ~mask;
+	    deactivate_channel (controller, channel);
+	    /*is_dma &= ~mask;*/
 	    return;
 	  }
 
@@ -1341,7 +1400,7 @@ void dma_process_block_mode_read (int controller, int channel)
 	    
 		if (dma[controller].i[channel].handler != NULL) {
 	      dma[controller].i[channel].handler (DMA_HANDLER_READ,
-				 length_transferred (controller, channel));
+						  amount_done);
 	  }
 	}
 }
@@ -1353,14 +1412,16 @@ void dma_process_block_mode_read (int controller, int channel)
 void dma_process_cascade_mode_read (int controller, int channel)
 {
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
 	  h_printf ("DMA: Attempt to use unsupported CASCADE mode\n");
 
-	is_dma &= ~mask;
+	  /*is_dma &= ~mask;*/
+	deactivate_channel (controller, channel);
+
 
 	if (dma[controller].i[channel].handler != NULL) {
 	    dma[controller].i[channel].handler (DMA_HANDLER_DONE, 0);
@@ -1377,10 +1438,10 @@ void dma_process_cascade_mode_read (int controller, int channel)
 void dma_process_verify_mode  (int controller, int channel)
 {
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
  	h_printf ("DMA: Attempt to use unsupported VERIFY direction by controller %d, channel %d\n",
 		  controller +1, channel);
@@ -1395,7 +1456,8 @@ void dma_process_verify_mode  (int controller, int channel)
 	dma[controller].i[channel].rfd = -1;
 	dma[controller].i[channel].wfd = -1;
 
-	is_dma &= ~mask;
+	deactivate_channel (controller, channel);
+	/*is_dma &= ~mask;*/
 }
 
 
@@ -1405,10 +1467,10 @@ void dma_process_verify_mode  (int controller, int channel)
 void dma_process_invalid_mode  (int controller, int channel)
 {
 	int ch;
-	int mask;
+	/*	int mask;*/
 
 	ch = get_ch (controller, channel);
-	mask = get_mask (ch);
+	/*	mask = get_mask (ch);*/
 
  	h_printf ("DMA: Attempt to use INVALID direction by controller %d, channel %d\n",
 		  controller +1, channel);
@@ -1423,7 +1485,8 @@ void dma_process_invalid_mode  (int controller, int channel)
 	dma[controller].i[channel].rfd = -1;
 	dma[controller].i[channel].wfd = -1;
 
-	is_dma &= ~mask;
+	deactivate_channel (controller, channel);
+	/*is_dma &= ~mask;*/
 }
 
 
