@@ -751,7 +751,7 @@ if (!(REG(eflags) & VIF)) return;
  *
  * Callers:
  * base/misc/ioctl.c
- * base/keyboard/n_serv_8042.c
+ * base/keyboard/serv_8042.c
  * base/keyboard/keyboard-server.c
  * base/serial/ser_irq.c
  * dosext/sound/sound.c
@@ -770,7 +770,9 @@ int do_irq()
 
     if(pic_ilevel==PIC_IRQ9)      /* unvectored irq9 just calls int 0x0a.. */
       if(!IS_REDIRECTED(intr)) {intr=0x0a;pic1_isr&= 0xffef;} /* & one EOI */
+#ifndef USE_NEW_INT
     if(IS_REDIRECTED(intr)||pic_ilevel<=PIC_IRQ1||in_dpmi)
+#endif /* not USE_NEW_INT */
     {
 #if 1 /* BUG CATCHER (if 1) */
 /* outputting more then one character here will change dynamic behave such that
@@ -782,7 +784,9 @@ g_printf("+%d",(int)pic_ilevel);
      if (in_dpmi) {
       run_pm_int(intr);
      } else {
+#ifndef USE_NEW_INT
       pic_cli();
+#endif /* not USE_NEW_INT */
       ssp = (unsigned char *)(LWORD(ss)<<4);
       sp = (unsigned long) LWORD(esp);
 
@@ -1206,10 +1210,6 @@ void pic_reset(void)
   pic_seti(PIC_IRQ0, timer_int_engine, 0);  /* do_irq0 in pic.c */
   pic_unmaski(PIC_IRQ0);
   pic_request(PIC_IRQ0);                    /* start timer */
-#ifndef NEW_KBD_CODE   
-  pic_seti(PIC_IRQ1, do_irq1, 0);           /* do_irq1 in dosio.c   */
-  pic_unmaski(PIC_IRQ1);
-#endif   
   if (mice->intdrv || mice->type == MOUSE_PS2) {
     pic_seti(PIC_IMOUSE, DOSEMUMouseEvents, 0);
     pic_unmaski(PIC_IMOUSE);
