@@ -6,6 +6,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <sys/utsname.h>
 
 #include "config.h"
 #include "emu.h"
@@ -17,12 +18,16 @@
 #include "memory.h"
 #include "bios.h"
 #include "kversion.h"
+#include "lpt.h"
 
 #include "dos2linux.h"
 #include "priv.h"
+#include "utilities.h"
 #ifdef X86_EMULATOR
 #include "cpu-emu.h"
 #endif
+#include "mhpdbg.h"
+
 
 /*
  * XXX - the mem size of 734 is much more dangerous than 704. 704 is the
@@ -37,9 +42,13 @@ struct debug_flags d =
 /* d  R  W  D  C  v  X  k  i  T  s  m  #  p  g  c  w  h  I  E  x  M  n  P  r  S */
 #endif
 
+int kernel_version_code = 0;
+int config_check_only = 0;
+
 static void     check_for_env_autoexec_or_config(void);
 int     parse_debugflags(const char *s, unsigned char flag);
 static void     usage(void);
+void memcheck_type_init(void);
 
 /*
  * DANG_BEGIN_FUNCTION cpu_override
@@ -82,8 +91,7 @@ int cpu_override (int cpu)
 static void
 config_defaults(void)
 {
-    FILE *fs;
-    char Line[256];
+    char *cpuflags;
     int k = 386;
 
     vm86s.cpu_type = CPU_386;
@@ -97,67 +105,45 @@ config_defaults(void)
     config.emuspeed = 50;	/* instruction cycles per us */
 #endif
 
-    fs=fopen("/proc/cpuinfo","r");
-    if (fs) {
-	while (fgets(Line,250,fs)) {
-	  /* get the CPU (processor#0) info */
-	  if (!strncmp(Line,"cpu",3) && isspace(Line[3])) {
-	    char *p = Line+4;
-	    while (*p && (*p!=':')) p++;
-
-	    /* if it's an x86, check which one */
-	    if ((sscanf(p+1,"%d",&k)==1) && ((k%100)==86)) {
-	      switch ((k/100)%10) {
-	        case 3: config.realcpu = CPU_386;	/* redundant */
-	        	break;
-	        case 5:
-	        case 6: config.realcpu = CPU_586;
-			/* bogospeed currently returns 0; should it deny
-			 * pentium features, fall back into 486 case */
-			if (!bogospeed(&config.cpu_spd, &config.cpu_tick_spd)) {
-			    config.pci = 1;	/* fair guess */
-			    config.rdtsc = 1;
-			    break;
-			}
-			k = 486;
-			/* fall thru */
-	        case 4: config.realcpu = CPU_486;
-	        	break;
-	        default:
-	        	exit(1);	/* no 186,286,786.. */
-	      }
-	    }
-	  }
-	  else if (!strncmp(Line,"fpu",3) && isspace(Line[3])) {
-	    /* get the fpu info. this is currently unused. */
-	    char *p = Line+4;
-	    while (*p && (*p!=':')) p++;
-	    if (*p) {
-	      p++; while (*p && isspace(*p)) p++;
-	      config.mathco = (*p=='y');
-	    }
-	  }
-	  else if (!strncmp(Line,"processor",9)) {
-	    /* if a 'processor:n' statement is found with n>0, meaning we
-	     * are on a SMP machine, deny use of pentium timer, force
-	     * use of kernel timing and stop parsing the file.
-	     */
-	    char *p = Line+9;
-	    while (*p && (*p!=':')) p++;
-	    if (*p) {
-	      p++; while (*p && isspace(*p)) p++;
-	      if (isdigit(*p) && (*p>'0') && config.rdtsc) {
-	  	fprintf(stderr,"Denying use of pentium timer on SMP machine\n");
-		config.smp = 1;		/* for checking overrides, later */
-	      	config.rdtsc = 0;
-	      	break;
-	      }
-	    }
-	  }
-	}
-	fclose(fs);
+    open_proc_scan("/proc/cpuinfo");
+    switch (get_proc_intvalue_by_key(
+          kernel_version_code > 0x20100+74 ? "cpu family" : "cpu" )) {
+      case 3: case 386: config.realcpu = CPU_386;	/* redundant */
+      	break;
+      case 5: case 586:
+      case 6: case 686:
+        config.realcpu = CPU_586;
+        config.pci = 1;	/* fair guess */
+        cpuflags = get_proc_string_by_key("flags");
+        if (strstr(cpuflags, "tsc")) {
+          /* bogospeed currently returns 0; should it deny
+           * pentium features, fall back into 486 case */
+          if (!bogospeed(&config.cpu_spd, &config.cpu_tick_spd)) {
+              config.rdtsc = 1;
+              break;
+          }
+        }
+        /* fall thru */
+      case 4: case 486: config.realcpu = CPU_486;
+      	break;
+      default:
+      	exit(1);	/* no 186,286,786.. */
     }
-    fprintf(stderr,"Running on CPU=%d86, FPU=%d\n",config.realcpu,config.mathco);
+    config.mathco = strcmp(get_proc_string_by_key("fpu"), "yes") == 0;
+    reset_proc_bufferptr();
+    k = 0;
+    while (get_proc_string_by_key("processor")) {
+      k++;
+      advance_proc_bufferptr();
+    }
+    if (k > 1) {
+      fprintf(stderr,"Denying use of pentium timer on SMP machine\n");
+      config.smp = 1;		/* for checking overrides, later */
+      config.rdtsc = 0;
+    }
+    close_proc_scan();
+    fprintf(stderr,"Running on CPU=%d86, FPU=%d, rdtsc=%d\n",
+      config.realcpu, config.mathco, config.rdtsc);
 
     config.hdiskboot = 1;	/* default hard disk boot */
 #ifdef X86_EMULATOR
@@ -216,6 +202,7 @@ config_defaults(void)
     config.X_font = "vga";
     config.usesX = 0;
     config.X = 0;
+    config.X_mgrab_key = "";	/* off , NULL = "Home" */
     config.hogthreshold = 10;	/* bad estimate of a good garrot value */
     config.chipset = PLAINVGA;
     config.cardtype = CARD_VGA;
@@ -259,11 +246,8 @@ config_defaults(void)
     config.dosbanner = 1;
     config.allowvideoportaccess = 0;
 
-    config.keyboard = KEYB_US;	/* What's the current keyboard  */
-    config.key_map = key_map_us;/* pointer to the keyboard-maps */
-    config.shift_map = shift_map_us;	/* Here the Shilt-map           */
-    config.alt_map = alt_map_us;/* And the Alt-map              */
-    config.num_table = num_table_dot;	/* Numeric keypad has a dot     */
+    config.keytable = &keytable_list[KEYB_USER]; /* What's the current keyboard  */
+
     config.detach = 0;		/* Don't detach from current tty and open
 				 * new VT. */
     config.debugout = NULL;	/* default to no debug output file */
@@ -296,11 +280,195 @@ config_defaults(void)
     config.sb_mixer = "/dev/mixer";
 #endif /* !__NetBSD__ */
     config.mpu401_base = 0x330;
+
+    config.vnet = 1;
+}
+
+void dump_config_status(void)
+{
+    char *s;
+    FILE *out = stderr;
+
+    fprintf(out, "\n-------------------------------------------------------------\n");
+    fprintf(out, "------dumping the runtime configuration _after_ parsing -----\n");
+    fprintf(out, "Version: dosemu-" VERSTR " versioncode = 0x%08x\n\n", DOSEMU_VERSION_CODE);
+    fprintf(out, "cpu ");
+    switch (vm86s.cpu_type) {
+      case CPU_386: s = "386"; break;
+      case CPU_486: s = "486"; break;
+      case CPU_586: s = "586"; break;
+      default:  s = "386"; break;
+    }
+    fprintf(out, "%s\nrealcpu ", s);
+    switch (config.realcpu) {
+      case CPU_386: s = "386"; break;
+      case CPU_486: s = "486"; break;
+      case CPU_586: s = "586"; break;
+      default:  s = "386"; break;
+    }
+    fprintf(out, "%s\n", s);
+    fprintf(out, "CPUclock %gMHz\ncpu_spd 0x%lx\ncpu_tick_spd 0x%lx\n",
+	(((double)LLF_US)/config.cpu_spd), config.cpu_spd, config.cpu_tick_spd);
+
+    fprintf(out, "pci %d\nrdtsc %d\nmathco %d\nsmp %d\n",
+                 config.pci, config.rdtsc, config.mathco, config.smp);
+#ifdef X86_EMULATOR
+    fprintf(out, "emuspeed %d\ncpuemu %d\n", config.emuspeed, config.cpuemu);
+#endif
+
+    fprintf(out, "hdiskboot %d\nmem_size %d\n",
+        config.hdiskboot, config.mem_size);
+    fprintf(out, "ems_size 0x%x\nems_frame 0x%x\nsecure %d\n",
+        config.ems_size, config.ems_frame, config.secure);
+    fprintf(out, "xms_size 0x%x\nmax_umb 0x%x\ndpmi 0x%x\n",
+        config.xms_size, config.max_umb, config.dpmi);
+    fprintf(out, "mouse_flag %d\nmapped_bios %d\nvbios_file %s\n",
+        config.mouse_flag, config.mapped_bios, (config.vbios_file ? config.vbios_file :""));
+    fprintf(out, "vbios_copy %d\nvbios_seg 0x%x\nvbios_size 0x%x\n",
+        config.vbios_copy, config.vbios_seg, config.vbios_size);
+    fprintf(out, "console %d\nconsole_keyb %d\nconsole_video %d\n",
+        config.console, config.console_keyb, config.console_video);
+    fprintf(out, "kbd_tty %d\nexitearly %d\n",
+        config.kbd_tty, config.exitearly);
+    fprintf(out, "fdisks %d\nhdisks %d\nbootdisk %d\n",
+        config.fdisks, config.hdisks, config.bootdisk);
+    fprintf(out, "term_esc_char 0x%x\nterm_color %d\nterm_updatefreq %d\n",
+        config.term_esc_char, config.term_color, config.term_updatefreq);
+    switch (config.term_charset) {
+      case CHARSET_LATIN: s = "latin"; break;
+      case CHARSET_IBM: s = "ibm"; break;
+      case CHARSET_FULLIBM: s = "fullibm"; break;
+      default: s = "(unknown)";
+    }
+    fprintf(out, "term_charset \"%s\"\nX_updatelines %d\nX_updatefreq %d\n",
+        s, config.X_updatelines, config.X_updatefreq);
+    fprintf(out, "X_display \"%s\"\nX_title \"%s\"\nX_icon_name \"%s\"\n",
+        (config.X_display ? config.X_display :""), config.X_title, config.X_icon_name);
+    fprintf(out, "X_blinkrate %d\nX_sharecmap %d\nX_mitshm %d\n",
+        config.X_blinkrate, config.X_sharecmap, config.X_mitshm);
+    fprintf(out, "X_fixed_aspect %d\nX_aspect_43 %d\nX_lin_filt %d\n",
+        config.X_fixed_aspect, config.X_aspect_43, config.X_lin_filt);
+    fprintf(out, "X_bilin_filt %d\nX_mode13fact %d\nX_winsize_x %d\n",
+        config.X_bilin_filt, config.X_mode13fact, config.X_winsize_x);
+    fprintf(out, "X_winsize_y %d\nX_gamma %d\nvgaemu_memsize 0x%x\n",
+        config.X_winsize_y, config.X_gamma, config.vgaemu_memsize);
+    fprintf(out, "vesamode_list %p\nX_lfb %d\nX_pm_interface %d\n",
+        config.vesamode_list, config.X_lfb, config.X_pm_interface);
+    fprintf(out, "X_keycode %d\nX_font \"%s\"\nusesX %d\n",
+        config.X_keycode, config.X_font, config.usesX);
+    fprintf(out, "X_mgrab_key \"%s\"\n",  config.X_mgrab_key);
+    switch (config.chipset) {
+      case PLAINVGA: s = "plainvga"; break;
+      case TRIDENT: s = "trident"; break;
+      case ET4000: s = "et4000"; break;
+      case DIAMOND: s = "diamond"; break;
+      case S3: s = "s3"; break;
+      case AVANCE: s = "avance"; break;
+      case ATI: s = "ati"; break;
+      case CIRRUS: s = "cirrus"; break;
+      case MATROX: s = "matrox"; break;
+      case WDVGA: s = "wdvga"; break;
+      default: s = "unknown"; break;
+    }
+    fprintf(out, "config.X %d\nhogthreshold %d\nchipset \"%s\"\n",
+        config.X, config.hogthreshold, s);
+    switch (config.cardtype) {
+      case CARD_VGA: s = "VGA"; break;
+      case CARD_MDA: s = "MGA"; break;
+      case CARD_CGA: s = "CGA"; break;
+      case CARD_EGA: s = "EGA"; break;
+      default: s = "unknown"; break;
+    }
+    fprintf(out, "cardtype \"%s\"\npci_video %d\nfullrestore %d\n",
+        s, config.pci_video, config.fullrestore);
+    fprintf(out, "graphics %d\ngfxmemsize %d\nvga %d\n",
+        config.graphics, config.gfxmemsize, config.vga);
+    switch (config.speaker) {
+      case SPKR_OFF: s = "off"; break;
+      case SPKR_NATIVE: s = "native"; break;
+      case SPKR_EMULATED: s = "emulated"; break;
+      default: s = "wrong"; break;
+    }
+    fprintf(out, "dualmon %d\nforce_vt_switch %d\nspeaker \"%s\"\n",
+        config.dualmon, config.force_vt_switch, s);
+    fprintf(out, "update %d\nfreq %d\nwantdelta %d\nrealdelta %d\n",
+        config.update, config.freq, config.wantdelta, config.realdelta);
+    fprintf(out, "timers %d\nkeybint %d\n",
+        config.timers, config.keybint);
+    fprintf(out, "tty_lockdir \"%s\"\ntty_lockfile \"%s\"\nconfig.tty_lockbinary %d\n",
+        config.tty_lockdir, config.tty_lockfile, config.tty_lockbinary);
+    fprintf(out, "num_ser %d\nnum_lpt %d\nnum_mice %d\nfastfloppy %d\n",
+        config.num_ser, config.num_lpt, config.num_mice, config.fastfloppy);
+    fprintf(out, "emusys \"%s\"\nemubat \"%s\"\nemuini \"%s\"\n",
+        (config.emusys ? config.emusys : ""), (config.emubat ? config.emubat : ""), (config.emuini ? config.emuini : ""));
+    fprintf(out, "dosbanner %d\nallowvideoportaccess %d\ndetach %d\n",
+        config.dosbanner, config.allowvideoportaccess, config.detach);
+    fprintf(out, "debugout \"%s\"\n",
+        (config.debugout ? config.debugout : (unsigned char *)""));
+    {
+	char buf[256];
+	GetDebugFlagsHelper(buf, 0);
+	fprintf(out, "debug_flags \"%s\"\n", buf);
+    }
+    {
+      extern void dump_keytable(FILE *f, struct keytable_entry *kt);
+      dump_keytable(out, config.keytable);
+    }
+    fprintf(out, "pre_stroke \"%s\"\n", (config.pre_stroke ? config.pre_stroke : (unsigned char *)""));
+    fprintf(out, "irqpassing= ");
+    if (config.sillyint) {
+      int i;
+      for (i=0; i <16; i++) {
+        if (config.sillyint & (1<<i)) {
+          fprintf(out, "IRQ%d", i);
+          if (config.sillyint & (0x10000<<i))
+            fprintf(out, "(sigio) ");
+          else
+            fprintf(out, " ");
+        }
+      }
+      fprintf(out, "\n");
+    }
+    else fprintf(out, "none\n");
+    fprintf(out, "must_spare_hardware_ram %d\n",
+        config.must_spare_hardware_ram);
+    {
+      int need_header_line =1;
+      int i;
+      for (i=0; i<(sizeof(config.hardware_pages)); i++) {
+        if (config.hardware_pages[i]) {
+          if (need_header_line) {
+            fprintf(out, "hardware_pages:\n");
+            need_header_line = 0;
+          }
+          fprintf(out, "%05x ", (i << 12) + 0xc8000);
+        }
+      }
+      if (!need_header_line) fprintf(out, "\n");
+    }
+    fprintf(out, "ipxsup %d\nvnet %d\npktflags 0x%x\n",
+	config.ipxsup, config.vnet, config.pktflags);
+    
+    {
+	int i;
+	struct printer *pptr;
+	extern struct printer lpt[NUM_PRINTERS];
+	for (i = 0, pptr = lpt; i < config.num_lpt; i++, pptr++) {
+	  fprintf(out, "LPT%d command \"%s  %s\"  timeout %d  device \"%s\"  baseport 0x%03x\n",
+	  i+1, pptr->prtcmd, pptr->prtopt, pptr->delay, (pptr->dev ? pptr->dev : ""), pptr->base_port); 
+	}
+    }
+
+    fprintf(out, "\nSOUND:\nsb_base 0x%x\nsb_dma %d\nsb_irq %d\nmpu401_base 0x%x\nsb_dsp \"%s\"\nsb_mixer \"%s\"\n",
+        config.sb_base, config.sb_dma, config.sb_irq, config.mpu401_base, config.sb_dsp, config.sb_mixer);
+    fprintf(out, "\n--------------end of runtime configuration dump -------------\n");
+    fprintf(out,   "-------------------------------------------------------------\n\n");
 }
 
 static void 
 open_terminal_pipe(char *path)
 {
+    PRIV_SAVE_AREA
     enter_priv_off();
     terminal_fd = DOS_SYSCALL(open(path, O_RDWR));
     leave_priv_setting();
@@ -336,6 +504,45 @@ open_Xmouse_pipe(char *path)
     return;
 }
 
+static void our_envs_init(char *usedoptions)
+{
+    struct utsname unames;
+    char *s;
+    char buf[256];
+    int i,j;
+
+    if (usedoptions) {
+        for (i=0,j=0; i<256; i++) {
+            if (usedoptions[i]) buf[j++] = i;
+        }
+        buf[j] = 0;
+        setenv("DOSEMU_OPTIONS", buf, 1);
+        return;
+    }
+    uname(&unames);
+    kernel_version_code = strtol(unames.release, &s,0) << 16;
+    kernel_version_code += strtol(s+1, &s,0) << 8;
+    kernel_version_code += strtol(s+1, &s,0);
+    sprintf(buf, "%d", kernel_version_code);
+    setenv("KERNEL_VERSION_CODE", buf, 1);
+    sprintf(buf, "%d", DOSEMU_VERSION_CODE);
+    setenv("DOSEMU_VERSION_CODE", buf, 1);
+    sprintf(buf, "%d", geteuid());
+    setenv("DOSEMU_EUID", buf, 1);
+    sprintf(buf, "%d", getuid());
+    setenv("DOSEMU_UID", buf, 1);
+}
+
+
+static void restore_usedoptions(char *usedoptions)
+{
+    char *p = getenv("DOSEMU_OPTIONS");
+    if (p) {
+        memset(usedoptions,0,256);
+        do usedoptions[(unsigned char)*p] = *p; while (*++p);
+    }
+}
+
 /*
  * DANG_BEGIN_FUNCTION config_init
  * 
@@ -349,18 +556,24 @@ open_Xmouse_pipe(char *path)
 void 
 config_init(int argc, char **argv)
 {
+    PRIV_SAVE_AREA
     extern char *commandline_statements;
     extern int dexe_running;
     int             c;
     char           *confname = NULL;
+    char           *dosrcname = NULL;
     char           *basename;
     char           *dexe_name = 0;
+    char usedoptions[256];
 
     if (argv[1] && !strcmp("--version",argv[1])) {
       usage();
       exit(0);
     }
 
+    memset(usedoptions,0,sizeof(usedoptions));
+    memcheck_type_init();
+    our_envs_init(0);
     config_defaults();
     basename = strrchr(argv[0], '/');	/* parse the program name */
     basename = basename ? basename + 1 : argv[0];
@@ -371,30 +584,58 @@ config_init(int argc, char **argv)
      * by a symbolic link called `xdos` which DOSEMU will use to switch
      * into X-mode. DANG_END_REMARK
      */
-    {
-	if (strcmp(basename, "xdos") == 0)
+    if (strcmp(basename, "xdos") == 0) {
 	    config.X = 1;	/* activate X mode if dosemu was */
+	    usedoptions['X'] = 'X';
 	/* called as 'xdos'              */
     }
 #endif
 
     opterr = 0;
-    confname = CONFIG_FILE;
-    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:VNtsgx:KL:m23456e:E:dXY:Z:o:Ou:")) != EOF) {
+    confname = CONFIG_SCRIPT;
+    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:VNtsgh:H:x:KL:m23456e:E:dXY:Z:o:Ou:")) != EOF) {
+	usedoptions[(unsigned char)c] = c;
 	switch (c) {
+	case 'h':
+	    config_check_only = atoi(optarg) + 1;
+	    break;
+	case 'H': {
+	    dosdebug_flags = strtoul(optarg,0,0) & 255;
+	    if (!dosdebug_flags) dosdebug_flags = DBGF_WAIT_ON_STARTUP;
+	    break;
+            }
 	case 'F':
 	    if (get_orig_uid()) {
+		FILE *f;
+		if (!get_orig_euid()) {
+		    /* we are running suid root as user */
+		    fprintf(stderr, "Sorry, -F option not allowed here\n");
+		    exit(1);
+		}
+		enter_priv_off();
+		f=fopen(optarg, "r");
+		leave_priv_setting();
+		if (!f) {
+		  fprintf(stderr, "Sorry, no access to configuration script %s\n", optarg);
+		  exit(1);
+		}
+		fclose(f);
+	    }
+	    confname = optarg;
+	    break;
+	case 'f':
+	    {
 		FILE *f;
 		enter_priv_off();
 		f=fopen(optarg, "r");
 		leave_priv_setting();
 		if (!f) {
-		  fprintf(stderr, "Sorry, no access to configuration file %s\n", optarg);
+		  fprintf(stderr, "Sorry, no access to user configuration file %s\n", optarg);
 		  exit(1);
 		}
 		fclose(f);
+	        dosrcname = optarg;
 	    }
-	    confname = optarg;	/* someone reassure me that this is *safe*? */
 	    break;
 	case 'L':
 	    dexe_name = optarg;
@@ -449,6 +690,8 @@ config_init(int argc, char **argv)
 	}
     }
 
+    if (config_check_only) d.config = 1;
+
     if (dexe_name || !strcmp(basename,"dosexec")) {
 	extern void prepare_dexe_load(char *name);
 	if (!dexe_name) dexe_name = argv[optind];
@@ -457,6 +700,7 @@ config_init(int argc, char **argv)
 	  exit(1);
 	}
 	prepare_dexe_load(dexe_name);
+	usedoptions['L'] = 'L';
     }
 
 #if defined(__NetBSD__) && defined(X_SUPPORT) && defined(X_GRAPHICS)
@@ -466,9 +710,11 @@ config_init(int argc, char **argv)
     }
 #endif
 
-    parse_config(confname);
+    our_envs_init(usedoptions);
+    parse_config(confname,dosrcname);
+    restore_usedoptions(usedoptions);
 
-    if (config.exitearly)
+    if (config.exitearly && !config_check_only)
 	leavedos(0);
 
     if (vm86s.cpu_type > config.realcpu) {
@@ -484,9 +730,19 @@ config_init(int argc, char **argv)
     optind = 0;
 #endif
     opterr = 0;
-    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:v:VNtT:sgx:KLm23456e:dXY:Z:E:o:Ou:")) != EOF) {
-	switch (c) {
+    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:v:VNtT:sgh:H:x:KLm23456e:dXY:Z:E:o:Ou:")) != EOF) {
+	/* Note: /etc/dosemu.conf may have disallowed some options
+	 *	 ( by removing them from $DOSEMU_OPTIONS ).
+	 *	 We skip them by re-checking 'usedoptions'.
+	 */
+	if (!usedoptions[(unsigned char)c]) {
+	    warn("command line option -%c disabled by dosemu.conf\n", c);
+	}
+	else switch (c) {
 	case 'F':		/* previously parsed config file argument */
+	case 'f':
+	case 'h':
+	case 'H':
 	case 'I':
 	case 'd':
 	case 'o':
@@ -553,6 +809,7 @@ config_init(int argc, char **argv)
 	    }
 	case 'D':
 	    parse_debugflags(optarg, 1);
+	    if (config_check_only) d.config = 1;
 	    break;
 	case 'P':
 	    if (terminal_fd == -1) {
@@ -625,15 +882,23 @@ config_init(int argc, char **argv)
 	}
     }
     if (config.X) {
-#ifndef NEW_KBD_CODE
 	if (!config.X_keycode) {
 	    extern void keyb_layout(int layout);
 	    keyb_layout(-1);
 	    c_printf("CONF: Forceing neutral Keyboard-layout, X-server will translate\n");
 	}
-#endif /* NEW_KBD_CODE */
 	config.console_video = config.vga = config.graphics = 0;
     }
+    else {
+	if (!can_do_root_stuff && config.console) {
+	    /* force use of Slang-terminal on console too */
+	    config.console = config.console_video = config.vga = config.graphics = 0;
+	    config.cardtype = 0;
+	    config.vbios_seg = 0;
+	    config.mapped_bios = 0;
+	    fprintf(stderr, "no console on low feature (non-suid root) DOSEMU\n");
+	}
+    }
     check_for_env_autoexec_or_config();
     if (under_root_login)  c_printf("CONF: running exclusively as ROOT:");
     else {
@@ -645,6 +910,12 @@ config_init(int argc, char **argv)
     }
     c_printf(" uid=%d (cached %d) gid=%d (cached %d)\n",
         geteuid(), get_cur_euid(), getegid(), get_cur_egid());
+
+    if (config_check_only) {
+	dump_config_status();
+	usage();
+	leavedos(0);
+    }
 }
 
 
@@ -698,7 +969,7 @@ check_for_env_autoexec_or_config(void)
 int parse_debugflags(const char *s, unsigned char flag)
 {
     char            c;
-
+    int ret = 0;
 #ifdef X_SUPPORT
     const char      allopts[] = "dRWDCvXkiTsm#pgcwhIExMnPrSe";
 #else
@@ -733,6 +1004,9 @@ int parse_debugflags(const char *s, unsigned char flag)
 	    break;
 	case 'D':		/* DOS int 21h */
 	    d.dos = flag;
+	    { extern void set_int21_revectored(int);
+	      set_int21_revectored(d.dos != 0);
+	    }
 	    break;
         case 'C':               /* CDROM */
 	    d.cdrom = flag;
@@ -813,7 +1087,7 @@ int parse_debugflags(const char *s, unsigned char flag)
 		newopts[0] = flag ? '+' : '-';
 		newopts[1] = 0;
 		strcat(newopts, allopts);
-#ifdef X86_EMULATOR
+#if 1 /* we need to _allways_ remove that flag here !!! #ifdef X86_EMULATOR */
 		/* hack-do not set 'e' flag if not explicitly specified */
 		{char *p=newopts; while (*p) {if (*p=='e') *p='g'; p++;}}
 #endif
@@ -828,20 +1102,21 @@ int parse_debugflags(const char *s, unsigned char flag)
 	default:
 	    fprintf(stderr, "Unknown debug-msg mask: %c\n\r", c);
 	    dbug_printf("Unknown debug-msg mask: %c\n", c);
-	    return 1;
+	    ret = 1;
 	}
-  return 0;
+  if (config_check_only) d.config = 1;
+  return ret;
 }
 
 static void
 usage(void)
 {
-    fprintf(stdout,
+    fprintf(stderr,
 	"dosemu-" VERSTR "\n\n"
 	"USAGE:\n"
-	"  dos  [-ABCckbVNtsgxKm23456ez] \\\n"
+	"  dos  [-ABCckbVNtsgxKm23456ez] [-h{0|1|2}] [-H dflags \\\n"
 	"       [-D flags] [-M SIZE] [-P FILE] [ {-F|-L} File ] \\\n"
-	"       [-o dbgfile] 2> vital_logs\n"
+	"       [-u confvar] [-f dosrcFile] [-o dbgfile] 2> vital_logs\n"
 	"  dos --version\n\n"
 	"    -2,3,4,5,6 choose 286, 386, 486 or 586 or 686 CPU\n"
 	"    -A boot from first defined floppy disk (A)\n"
@@ -866,10 +1141,14 @@ usage(void)
 	"       p=printer r=pic      s=serial v=video     w=warning x=xms\n"
 	"    -E STRING pass DOS command on command line\n"
 	"    -e SIZE enable SIZE K EMS RAM\n"
-	"    -F use config-file File\n"
+	"    -F use File as global config-file\n"
+	"    -f use dosrcFile as user config-file\n"
 	"    -L load and execute DEXE File\n"
 	"    -I insert config statements (on commandline)\n"
 	"    -g enable graphics modes (!%%#)\n"
+	"    -h dump configuration to stderr and exit (sets -D+c)\n"
+	"       0=no parser debug, 1=loop debug, 2=+if_else debug\n"
+	"    -H wait for dosdebug terminal at startup and pass dflags\n"
 	"    -K Do int9 (!#)\n"
 	"    -k use PC console keyboard (!)\n"
 	"    -M set memory size to SIZE kilobytes (!)\n"
@@ -880,6 +1159,7 @@ usage(void)
 	"    -P copy debugging output to FILE\n"
 	"    -T DIR set tmpdir\n"
 	"    -t try new timer code (#)\n"
+	"    -u set user configuration variable 'confvar' prefixed by 'u_'.\n"
 	"    -V use BIOS-VGA video modes (!#%%)\n"
 	"    -v NUM force video card type\n"
 	"    -x SIZE enable SIZE K XMS RAM\n"
