@@ -26,6 +26,7 @@
 #include "disks.h"
 #include "priv.h"
 
+static int disks_initiated = 0;
 
 #define FDISKS config.fdisks
 #define HDISKS config.hdisks
@@ -352,6 +353,7 @@ hdisk_auto(struct disk *dp)
 void
 partition_setup(struct disk *dp)
 {
+  PRIV_SAVE_AREA
   int part_fd, i;
   unsigned char tmp_mbr[SECTOR_SIZE];
   char *hd_name;
@@ -522,6 +524,7 @@ disk_close(void)
 {
   struct disk *dp;
 
+  if (!disks_initiated) return;  /* just to be safe */
   for (dp = disktab; dp < &disktab[FDISKS]; dp++) {
     if (dp->removeable && dp->fdesc >= 0) {
       d_printf("DISK: Closing a disk\n");
@@ -535,6 +538,7 @@ disk_close(void)
 void
 disk_open(struct disk *dp)
 {
+  PRIV_SAVE_AREA
   struct fd_type fl;
 
   if (dp == NULL || dp->fdesc >= 0)
@@ -606,6 +610,7 @@ static void sigalarm_onoff(int on)
 void
 disk_open(struct disk *dp)
 {
+  PRIV_SAVE_AREA
   struct floppy_struct fl;
 
   if (dp == NULL || dp->fdesc >= 0)
@@ -615,7 +620,7 @@ disk_open(struct disk *dp)
   dp->fdesc = SILENT_DOS_SYSCALL(open(dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
   leave_priv_setting();
 
-  if (dp->fdesc < 0) 
+  if (dp->fdesc < 0) {
     if (errno == EROFS || errno == ENODEV) {
       enter_priv_on();
       dp->fdesc = DOS_SYSCALL(open(dp->dev_name, O_RDONLY, 0));
@@ -637,6 +642,7 @@ disk_open(struct disk *dp)
 #endif
       return;
     }
+  }
   else dp->rdonly = dp->wantrdonly;
 
 {
@@ -684,10 +690,12 @@ void
 disk_close_all(void)
 {
   struct disk *dp;
-
+  
+  if (!disks_initiated) return;  /* prevent idiocy */
   if (config.bootdisk && bootdisk.fdesc >= 0) {
+    d_printf("Boot disk Closing %x\n", bootdisk.fdesc);
     (void) close(bootdisk.fdesc);
-    bootdisk.fdesc = 0;
+    bootdisk.fdesc = -1;
     d_printf("BOOTDISK Closing\n");
   }
   for (dp = disktab; dp < &disktab[FDISKS]; dp++) {
@@ -704,6 +712,7 @@ disk_close_all(void)
       dp->fdesc = -1;
     }
   }
+  disks_initiated = 0;
 }
 
 /*
@@ -717,6 +726,7 @@ disk_close_all(void)
 void
 disk_init(void)
 {
+  PRIV_SAVE_AREA
   struct disk *dp=NULL;
   struct stat stbuf;
 
@@ -726,12 +736,13 @@ disk_init(void)
 
 #endif
 
+  disks_initiated = 1;  /* disk_init has been called */
   if (config.bootdisk) {
     enter_priv_on();
     bootdisk.fdesc = open(bootdisk.dev_name,
 			  bootdisk.rdonly ? O_RDONLY : O_RDWR, 0);
     leave_priv_setting();
-    if (bootdisk.fdesc < 0) 
+    if (bootdisk.fdesc < 0) {
       if (errno == EROFS) {
         enter_priv_on();
         bootdisk.fdesc = open(bootdisk.dev_name, O_RDONLY, 0);
@@ -747,6 +758,7 @@ disk_init(void)
         error("can't open bootdisk %s: %sn", dp->dev_name, strerror(errno));
         leavedos(23);
       }
+    }
     else bootdisk.rdonly = bootdisk.wantrdonly;
     bootdisk.removeable = 0;
   }
@@ -780,7 +792,7 @@ disk_init(void)
     enter_priv_on();
     dp->fdesc = open(dp->dev_name, dp->rdonly ? O_RDONLY : O_RDWR, 0);
     leave_priv_setting();
-    if (dp->fdesc < 0) 
+    if (dp->fdesc < 0) {
       if (errno == EROFS || errno == EACCES) {
         enter_priv_on();
         dp->fdesc = open(dp->dev_name, O_RDONLY, 0);
@@ -796,6 +808,7 @@ disk_init(void)
         error("can't open %s: %s\n", dp->dev_name, strerror(errno));
         leavedos(25);
       }
+    }
     else dp->rdonly = dp->wantrdonly;
   }
   for (dp = hdisktab; dp < &hdisktab[HDISKS]; dp++) {
@@ -1391,7 +1404,7 @@ int13(u_char i)
     }
 
     if (dp->rdonly) {
-      error("ERROR: write protect!\n");
+      d_printf("DISK is write protected!\n");
       show_regs(__FILE__, __LINE__);
       if (dp->removeable)
 	HI(ax) = DERR_WP;
@@ -1534,7 +1547,7 @@ void background_ioctl_thread(int start)
   if (!start) {
     /* we come here _before_ any thread is started
      * but threading is active and we run at scope of thread0.
-     * We now create the resoeurce we need.
+     * We now create the resource we need.
      */
      ioctlmbox_in = create_local_mailbox("diskioctl_in", 2);
      ioctlmbox_out = create_local_mailbox("diskioctl_out", 2);
