@@ -30,9 +30,7 @@
 #include "vc.h"
 #include "port.h"
 
-#ifdef NEW_KBD_CODE
 #include "keyboard.h"
-#endif
 
 #include "priv.h"
 
@@ -51,8 +49,21 @@ extern void X_change_mouse_cursor(void);
 void DOSEMUSetupMouse(void);
 
 #define Mouse_INT	(0x33 * 16)
+#ifndef USE_NEW_INT
 /* I have not properly tested this INT74 - JES 96/10/20 */
 #define Mouse_INT74	(0x74 * 16)
+#else /* USE_NEW_INT */
+/* DANG_BEGIN_REMARK
+ * I have not properly tested this INT74 - JES 96/10/20 
+ * I have removed it.  INT74 is irq 12.  Which I suppose is the proper
+ * irq for a ps2 mouse.  It appears initial support was planned to
+ * support irq 12 and at Mouse_ROUTINE_OFF is a routine that
+ * acknowledges an irq.  That routine is probably what should be
+ * acknowledging irq12, and what int 0x74 should point to.  
+ * I have disabled int0x74 support for now. --EB 29 Oct 1997
+ * DANG_END_REMARK
+ */
+#endif /* USE_NEW_INT */
 
 static
 void mouse_cursor(int), mouse_pos(void), mouse_setpos(void),
@@ -152,7 +163,13 @@ mouse_helper(void)
   case 0:				/* Reset iret for mouse */
     m_printf("MOUSE move iret !\n");
     SETIVEC(0x33, Mouse_SEG, Mouse_INT);
+#ifndef USE_NEW_INT
     SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
+#else /* USE_NEW_INT */
+#if 0
+    SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
+#endif
+#endif /* USE_NEW_INT */
     break;
   case 1:				/* Select Microsoft Mode */
     m_printf("MOUSE Microsoft Mouse (two buttons) selected.\n");
@@ -571,7 +588,13 @@ mouse_software_reset(void)
   mouse.cs=0;
   mouse.ip=0;
   SETIVEC(0x33, Mouse_SEG, Mouse_INT);
+#ifndef USE_NEW_INT
   SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
+#else /* USE_NEW_INT */
+#if 0
+  SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
+#endif
+#endif /* USE_NEW_INT */
 
 #ifdef X_SUPPORT
 #ifndef X_SLOW_CHANGE_CURSOR
@@ -734,6 +757,12 @@ mouse_reset_to_current_video_mode(void)
    * confuse people. Besides, they ignore the ACTUAL maxx,maxy values as
    * stored in the BIOS variables.
    */
+  /* To get quicken to run correctly I no longer ignore maxx and maxy
+   * values as stored in the BIOS variables for text mode.
+   * Does this violate a spec?
+   * -- EB 25 May 1998
+   */
+
 
   if (current_video.textgraph == 'T') {
     mouse.gfx_cursor = FALSE;
@@ -781,7 +810,13 @@ mouse_reset(int flag)
 {
   m_printf("MOUSE: reset mouse/installed!\n");
   SETIVEC(0x33, Mouse_SEG, Mouse_INT);
+#ifndef USE_NEW_INT
   SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
+#else /* USE_NEW_INT */
+#if 0
+  SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
+#endif
+#endif /* USE_NEW_INT */
 
   /* Return 0xffff on mouse installed, 0x0000 - no mouse driver installed */
   /* Return number of mouse buttons */
@@ -1106,23 +1141,17 @@ mouse_enable_internaldriver()
 {
   mice->intdrv = TRUE;
   SETIVEC(0x33, Mouse_SEG, Mouse_INT);
+#ifndef USE_NEW_INT
   SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
+#else /* USE_NEW_INT */
+#if 0
+  SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
+#endif
+#endif /* USE_NEW_INT */
   m_printf("MOUSE: Enable InternalDriver\n");
 }
 
 
-#ifndef NEW_KBD_CODE
-/* for the new keyboard code, these are defined in keyboard.h.
- * The old keyboard code doesn't have symbolic constants, so we
- * define them here
- */
-#define KEY_LEFT  0x4b
-#define KEY_RIGHT 0x4d
-#define KEY_UP    0x48
-#define KEY_DOWN  0x50
-#define KEY_HOME  0x47
-#define KEY_END   0x4f
-#endif
 
 /* XXX - something's wrong here. Shouldn't the first 4 cases move the cursor?
  */
@@ -1200,6 +1229,13 @@ mouse_move(void)
         default: mouse.xshift = 0; break;
   }
 
+#ifdef NEW_X_MOUSE
+  /*
+   * it looks as if mouse.yshift is not set otherwise -- 1998/02/22 sw
+   */
+  if(current_video.textgraph != 'T') mouse.yshift = 0;
+#endif /* NEW_X_MOUSE */
+
   mouse.maxx = mouse_roundx(mouse.maxx - 1);
   mouse.maxy = mouse_roundy(mouse.maxy - 1);
   m_printf("maxx=%i, maxy=%i\n", mouse.maxx, mouse.maxy);
@@ -1358,7 +1394,13 @@ mouse_event()
     if(in_dpmi && !in_dpmi_dos_int 
           && !((mouse.cs == DPMI_SEG) && ((void *)(Bit32u)mouse.ip == DPMI_mouse_callback)))
                   fake_pm_int();
+
+    /* push iret frame on _SS:_SP. At F000:20F7 (bios.S) we get an
+     * iret and return to _CS:_IP */
     fake_int();
+
+    /* push all 16-bit regs plus _DS,_ES. At F000:20F4 (bios.S) we find
+     * 'pop es; pop ds; popa' */
     fake_pusha();
 
     LWORD(eax) = mouse_events;
@@ -1370,8 +1412,10 @@ mouse_event()
     if (mouse.threebuttons)
       LWORD(ebx) |= (mouse.mbutton ? 4 : 0);
 
+    /* push return address F000:20F4 */
     fake_call(Mouse_SEG, Mouse_OFF + 4);	/* skip to popa */
 
+    /* jump to mouse cs:ip */
     REG(cs) = mouse.cs;
     REG(eip) = mouse.ip;
 
@@ -1415,10 +1459,8 @@ static void
 mouse_update_cursor(void)
 {
 	/* sigh, too many programs seem to expect the mouse cursor
-		to magically redraw itself when in text mode, so we'll
-		bend to their will... */
-	if ((mouse.rx != mouse.oldrx || mouse.ry != mouse.oldry) &&
-			!mouse.gfx_cursor) {
+		to magically redraw itself, so we'll bend to their will... */
+	if (mouse.rx != mouse.oldrx || mouse.ry != mouse.oldry) {
 		mouse_do_cur();
 		mouse.oldrx = mouse.rx;
 		mouse.oldry = mouse.ry;
@@ -1524,6 +1566,7 @@ mouse_sethandler(void *f, us * cs, us * ip)
 void
 mouse_init(void)
 {
+  PRIV_SAVE_AREA
   serial_t *sptr=NULL;
   char mouse_ver[]={2,3,4,5,0x14,0x7,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f};
 #if 1 /* BUG CATCHER */
