@@ -1,5 +1,6 @@
 /* DANG_BEGIN_MODULE
  * 
+ * REMARK
  * ser_ports.c: Serial ports for DOSEMU: Software emulated 16550 UART!
  * Please read the README.serial file in this directory for more info!
  * 
@@ -14,6 +15,7 @@
  *      marky@magmacom.com
  *      ag115@freenet.carleton.ca
  *
+ * /REMARK
  * DANG_END_MODULE
  */
 
@@ -118,7 +120,7 @@ void uart_fill(int num)
 
   /* Is it time to do another read() of the serial device yet? 
    * The rx_timer is used to prevent system load caused by empty read()'s 
-   * It also skip the following code block if the recieve buffer 
+   * It also skip the following code block if the receive buffer 
    * contains enough data for a full FIFO (at least 16 bytes).
    * The receive buffer is a sliding buffer.
    */
@@ -151,7 +153,7 @@ void uart_fill(int num)
       com[num].LSR |= UART_LSR_DR;		/* Set recv data ready bit */
 
       /* The following code is to emulate the 16 byte (configurable)
-       * limitation of the recieve FIFO, for compatibility purposes.
+       * limitation of the receive FIFO, for compatibility purposes.
        * Reset the receive FIFO counter up to a value of 16 (configurable)
        * if we are not within an interrupt.
        */
@@ -162,7 +164,7 @@ void uart_fill(int num)
           com[num].rx_fifo_bytes = com[num].rx_buf_bytes;
       }
       
-      /* Has it gone above the recieve FIFO trigger level? */
+      /* Has it gone above the receive FIFO trigger level? */
       if (com[num].rx_buf_bytes >= com[num].rx_fifo_trigger) {
         if(s3_printf) s_printf("SER%d: Func uart_fill requesting RX_INTR\n",num);
         com[num].LSRqueued |= UART_LSR_DR;	/* Update queued LSR */
@@ -420,20 +422,20 @@ static int get_rx(int num)
   static int val;
   com[num].rx_timeout = TIMEOUT_RX;		/* Reset timeout counter */
 
-  /* If the Recieved-Data-Ready bit is clear then return a 0
+  /* If the Received-Data-Ready bit is clear then return a 0
    * since we're not supposed to read from the buffer right now!
    */
   if ( !(com[num].LSR & UART_LSR_DR)) return 0;
   
   /* Is the FIFO enabled? */
   if (com[num].fifo_enable) {
-    /* Read data from the recieve buffer.  The 16-byte (configurable)
-     * limitation of the FIFO is emulated (even though the recieve
+    /* Read data from the receive buffer.  The 16-byte (configurable)
+     * limitation of the FIFO is emulated (even though the receive
      * buffer is bigger) for compatibility purposes.  Note, that the 
      * following code is optimized for speed rather than compactness.
      */
      
-    /* Is the recieve FIFO empty? */
+    /* Is the receive FIFO empty? */
     if (com[num].rx_fifo_bytes == 0) return 0;
 
     val = com[num].rx_buf[com[num].rx_buf_start];	/* Get byte */
@@ -469,7 +471,7 @@ static int get_rx(int num)
         com[num].IIR = (com[num].IIR & UART_IIR_ID) | UART_IIR_FIFO;
       }
     }
-    return val;		/* Return recieved byte */
+    return val;		/* Return received byte */
   }
   
   /* The following code executes only if in non-FIFO mode */
@@ -494,7 +496,7 @@ static int get_rx(int num)
     com[num].rx_buf_bytes--;
   }
   
-  return val;		/* Return recieved byte */
+  return val;		/* Return received byte */
 }
 
 
@@ -591,7 +593,7 @@ static void put_tx(int num, int val)
   com[num].TX = val;			/* Mainly used in overflow cases */
   com[num].tx_trigger = 1;		/* Time to trigger next tx int */
   com[num].int_condition &= ~TX_INTR;	/* TX interrupt condition satisifed */
-  com[num].LSR &= ~(UART_LSR_TEMT | UART_LSR_THRE);	/* THR not empty */
+  com[num].LSR &= ~UART_LSR_TEMT;	/* TEMT not empty */
 
   /* If Transmit interrupt status is set, then update interrupt status */
   if ((com[num].IIR & UART_IIR_ID) == UART_IIR_THRI)
@@ -660,7 +662,9 @@ static void put_tx(int num, int val)
   /* Else, not in loopback mode */
   
   if (com[num].fifo_enable) {			/* Is FIFO enabled? */
-    if (com[num].tx_buf_bytes == TX_BUFFER_SIZE) {	/* Is FIFO full? */
+    if (com[num].tx_buf_bytes >= TX_BUFFER_SIZE) {	/* Is FIFO already full? */
+      /* try to write the character directly to the tty, hoping it
+       * will be queued there. Hmmm... */
       rtrn = RPT_SYSCALL(write(com[num].fd,&com[num].tx_buf[com[num].tx_buf_start],1));
       if (rtrn != 1) {				/* Did transmit fail? */
         com[num].tx_overflow = 1;		/* Set overflow flag */
@@ -686,6 +690,9 @@ static void put_tx(int num, int val)
     if (rtrn != 1) 				/* Did transmit fail? */
       com[num].tx_overflow = 1; 		/* Set overflow flag */
   }
+  if (!com[num].fifo_enable ||
+       com[num].tx_buf_bytes >= (TX_BUFFER_SIZE-1)) 	/* Is FIFO full? */
+      com[num].LSR &= ~UART_LSR_THRE;		/* THR full */
 }
 
 
@@ -917,7 +924,7 @@ put_msr(int num, int val)
  * DANG_END_FUNCTION
  */
 int
-do_serial_in(int num, int address)
+do_serial_in(int num, ioport_t address)
 {
   static int val;
   switch (address - com[num].base_port) {
@@ -1000,7 +1007,7 @@ do_serial_in(int num, int address)
  * DANG_END_FUNCTION
  */
 int
-do_serial_out(int num, int address, int val)
+do_serial_out(int num, ioport_t address, int val)
 {
   switch (address - com[num].base_port) {
   case UART_TX:		/* Write to Transmit Holding Register */
