@@ -1,6 +1,6 @@
 /* a port of CMDLINE.AWK */
-#ifndef __LARGE__
-#error "Use the LARGE model!"
+#if !defined( __LARGE__) && !defined(__COMPACT__)
+#error "Use the LARGE or COMPACT model!"
 #endif
 
 #include <stdio.h>
@@ -8,6 +8,7 @@
 #include <string.h>
 #include <io.h>
 #include <dos.h>
+#include "detect.h"
 
 int msetenv(char *,char *);
 
@@ -16,12 +17,18 @@ int msetenv(char *,char *);
 main()
 {
 	char *buff = malloc(CMDBUFFSIZE), *p, *q, *endb;
+
+  if (!is_dosemu()) {
+    printf("This program requires DOSEMU to run, aborting\n");
+    exit(1);
+  }
+
 	if (!buff)
 	    perror("malloc failure"),
 	    exit(1);
 	*(endb = buff + read(0,buff,CMDBUFFSIZE)) = '\0';
 	for (p = buff; p < endb; p = q + strlen(q)+1)
-		 if (*p != '-' && (q = strchr(p,'='))) {
+		 if (*p != '-' && ((q = strchr(p,'='))!=0)) {
 		     *q++ = '\0';
 		     if (msetenv(p,q))
 			 exit(1);
