@@ -30,6 +30,10 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
+#ifndef ENOMEDIUM
+  /* kernels < 2.0.32 don't have this defined */
+  #define ENOMEDIUM       123     /* No medium found */
+#endif
 #ifdef __linux__
 #include <linux/cdrom.h>
 #endif
@@ -200,6 +204,7 @@ void dump_cd_sect (char *tb)
 
 void cdrom_reset()
 {
+  PRIV_SAVE_AREA
   /* after a disk change a new read access will
      return an error. In order to unlock this condition
      the drive must be reopened.
@@ -222,6 +227,7 @@ void cdrom_reset()
 
 void cdrom_helper(void)
 {
+   PRIV_SAVE_AREA
    unsigned char *req_buf,*transfer_buf;
    unsigned int Sector_plus_150,Sector;
    struct cdrom_msf cdrom_msf;
@@ -318,11 +324,13 @@ void cdrom_helper(void)
                 if (cdrom_fd < 0) {
 		  C_printf("CDROM: cdrom open (%s) failed: %s\n",
 			    path_cdrom, strerror(error));
-                  if (error == EIO) {
+                  LO(ax) = 0;
+                  if ((error == EIO) || (error==ENOMEDIUM)) {
                     /* drive which cannot be opened if no
                        disc is inserted!                   */
-                    LO(ax) = 0;
                     cdu33a = 1;
+                    if (! eject_allowed)
+                       LO(ax) = 1; /* no disk in drive */
                    }
                   else LO(ax) = 1; /* no cdrom drive installed */
                   if (! eject_allowed)
@@ -526,14 +534,15 @@ void cdrom_helper(void)
      case 0x0A: /* device status */
                 HI(ax) = 0; LO(ax) = 0;
                 if (eject_allowed) {
-                  if (ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl))
+                  if (ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl)) {
                     if (ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl))
                       { /* no disk in drive */
                         LWORD(ebx) = audio_status.status | 0x800;
 			C_printf("CDROM: subch failed: %s\n", strerror(errno));
                         break;
                       }
-                     else cdrom_reset();
+                    else cdrom_reset();
+		  }
                 }
                 /* disk in drive */
                 LWORD(ebx) = audio_status.status;
