@@ -783,6 +783,7 @@ init_drive(int dd, char *path, char *options)
 int
 mfs_redirector(void)
 {
+  PRIV_SAVE_AREA
   int dos_fs_redirect();
   int ret;
 
@@ -819,6 +820,7 @@ mfs_redirector(void)
 int
 mfs_inte6(void)
 {
+  PRIV_SAVE_AREA
   boolean_t dos_fs_dev();
   boolean_t result;
 
@@ -3315,7 +3317,7 @@ dos_fs_redirect(state)
     return TRUE;
   case SEEK_FROM_EOF:		/* 0x21 */
     {
-      int offset = (state->ecx << 16) + (state->edx);
+      int offset = (int)WORD(state->ecx << 16) + WORD(state->edx);
 
       fd = sft_fd(sft);
       Debug0((dbg_fd, "Seek From EOF fd=%x ofs=%d\n",
@@ -3327,8 +3329,8 @@ dos_fs_redirect(state)
 	      fd, offset));
       if (offset != -1) {
 	sft_position(sft) = offset;
-	state->edx = offset >> 16;
-	state->eax = WORD(offset);
+	SETWORD(&(state->edx), offset >> 16);
+	SETWORD(&(state->eax), WORD(offset));
 	return (TRUE);
       }
       else {
@@ -3380,7 +3382,14 @@ dos_fs_redirect(state)
 		struct flock larg;
 		unsigned long mask = 0xC0000000;
 
+#if 1	/* fix for foxpro problems with lredired drives from
+         * Sergey Suleimanov <solt@atibank.astrakhan.su>.
+         * Needs more testing --Hans 98/01/30
+	 */
+		larg.l_type = is_lock ? F_RDLCK : F_UNLCK;
+#else
 		larg.l_type = is_lock ? F_WRLCK : F_UNLCK;
+#endif
 		larg.l_whence = SEEK_SET;
 		larg.l_start = pt->offset;
 		larg.l_len = pt->size;
@@ -3489,6 +3498,7 @@ dos_fs_redirect(state)
       if (((action & 0xf) == 1) && file_exists) {
 	/* Open if does exist */
 	SETWORD(&(state->ecx), 0x1);
+	dos_mode = mode & 0xF;
 	goto do_open_existing;
       }
 
