@@ -15,13 +15,13 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/vm86.h>
-#include <linux/sockios.h>
-#include <asm/types.h>
-#include <linux/ipx.h>
-#ifndef IPX_TYPE
-  #define IPX_TYPE 1   /* glibc's headers seem missing that
-                          (normally defined in <linux/socket.h>) */
-#endif
+#if defined (__GLIBC__) && __GLIBC__ >=2
+#  include <netipx/ipx.h>
+#else
+#  include <linux/sockios.h>
+#  include <asm/types.h>
+#  include <linux/ipx.h>
+#endif /* __GLIBC >= 2 */
 #include <netinet/in.h>
 #include <errno.h>
 
@@ -403,6 +403,7 @@ printIPXHeader(IPXPacket_t * IPXHeader)
 u_char
 IPXOpenSocket(u_short port, u_short * newPort)
 {
+  PRIV_SAVE_AREA
   int sock;			/* sock here means Linux socket handle */
   int opt;
   struct sockaddr_ipx ipxs;
