@@ -16,19 +16,12 @@
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
-#if __GLIBC__ > 1
-  #include <asm/types.h>
-  #include <asm/sockios.h>
-  #include <linux/ipx.h>
-  #ifndef IPX_TYPE
-    #define IPX_TYPE 1   /* glibc's headers seem missing that
-                            (normally defined in <linux/socket.h>) */
-  #endif
-  #include <net/route.h>
+#include <net/route.h>
+#if defined (__GLIBC__) && __GLIBC__ >= 2
+  #include<netipx/ipx.h>
 #else
   #include <linux/sockios.h>
   #include <linux/ipx.h>
-  #include <linux/route.h>
 #endif
 #include <netinet/in.h>
 #include <errno.h>
@@ -51,6 +44,7 @@ typedef struct
 int AddRoute( unsigned long targetNet, unsigned network,
         unsigned char node[] )
 {
+	PRIV_SAVE_AREA
 	struct rtentry rt;
 	struct sockaddr_ipx	*st = (struct sockaddr_ipx *)&rt.rt_dst;
 	struct sockaddr_ipx	*sr = (struct sockaddr_ipx *)&rt.rt_gateway;
@@ -89,6 +83,7 @@ int AddRoute( unsigned long targetNet, unsigned network,
 /* returns 0 on success, less than 0 on failure */
 int IPXGetLocalTarget( unsigned long network, int *hops, int *ticks )
 {
+	PRIV_SAVE_AREA
  	int sock;
 	struct sockaddr_ipx ipxs;
 	int opt=1;
