@@ -76,9 +76,11 @@ static unsigned long TRs[2] =
  */
 int cpu_trap_0f (unsigned char *csp, struct sigcontext_struct *scp)
 {
-  	g_printf("CPU: TRAP op 0F %02x %02x\n",csp[1],csp[2]);
-
   	if (in_dpmi && (scp==NULL)) return 0; /* for safety */
+	/* moved access to "csp" after safety check,
+	 *                          -- 980627 Andreas Kirschbaum
+	 */
+	g_printf("CPU: TRAP op 0F %02x %02x\n",csp[1],csp[2]);
 
 	if (csp[1] == 0x06) {
 		(in_dpmi? scp->eip:LWORD(eip)) += 2;  /* CLTS - ignore */
@@ -294,6 +296,7 @@ int
 allow_io(unsigned int start, int size, int permission, int ormask, int andmask,
 unsigned int portspeed, char *device)
 {
+  PRIV_SAVE_AREA
   FILE *fp;
   int opendevice= -1;
   char line[100];
@@ -493,6 +496,7 @@ port_writeable(unsigned short port)
 unsigned char
 read_port(unsigned short port)
 {
+  PRIV_SAVE_AREA
   unsigned char r;
   int i = find_port(port, IO_READ);
 
@@ -532,6 +536,7 @@ read_port(unsigned short port)
 unsigned int
 read_port_w(unsigned short port)
 {
+  PRIV_SAVE_AREA
   unsigned int r;
   int i = find_port(port,IO_READ);
   int j = find_port(port+1,IO_READ);
@@ -590,6 +595,7 @@ read_port_w(unsigned short port)
 int
 write_port(unsigned int value, unsigned short port)
 {
+  PRIV_SAVE_AREA
   int i = find_port(port, IO_WRITE);
 
   if (i == -1)
@@ -628,6 +634,7 @@ write_port(unsigned int value, unsigned short port)
 int
 write_port_w(unsigned int value,unsigned short port)
 {
+  PRIV_SAVE_AREA
   int i = find_port(port, IO_WRITE);
   int j = find_port(port + 1, IO_WRITE);
 
@@ -677,7 +684,7 @@ write_port_w(unsigned int value,unsigned short port)
 
 void safe_port_out_byte(const unsigned short port, const unsigned char byte)
 {
-        if(i_am_root)  {
+        if(can_do_root_stuff)  {
                 int result;
 
                 result = set_ioperm(port, 1, 1);
@@ -695,7 +702,7 @@ char safe_port_in_byte(const unsigned short port)
 {
         unsigned char value=0;
 
-        if(i_am_root) {
+        if(can_do_root_stuff) {
                 int result;
 
                 result = set_ioperm(port, 1, 1);
@@ -705,7 +712,7 @@ char safe_port_in_byte(const unsigned short port)
 		if (result) error("failed to disable port %x\n", port);
         }       else i_printf("want to ");
         i_printf("in(%x)", port);
-        if(i_am_root)
+        if(can_do_root_stuff)
                 i_printf(" = 0x%x", value);
         i_printf("\n");
         return value;
