@@ -95,8 +95,8 @@ static unsigned char port_handles;	/* number of io_handler's */
 
 static const char *irq_handler_name[EMU_MAX_IRQS];
 
-#define SET_HANDLE(p,h)		port_handle_table[(p)]=(h)
-#define EMU_HANDLER(port)	port_handler[port_handle_table[(port)]]
+#define SET_HANDLE(p,h)		port_handle_table[(Bit16u)(p)]=(h)
+#define EMU_HANDLER(port)	port_handler[port_handle_table[(Bit16u)(port)]]
 
 /* This is the same code that was used in cpu.c, priv_iopl is really
    very slow */
@@ -607,10 +607,12 @@ static Bit8u special_port_inb(ioport_t port)
 		return ((res & 0xfd) | (hgc_Konv & 0x02));
 	    }
 	}
+#if X_GRAPHICS
 	if ((port==0x3ba)||(port==0x3da)) {
 		res = Attr_get_input_status_1();
 	}
 	else
+#endif
 	if (port==0x3db)	/* light pen strobe reset */
 		res = 0;
 	return res;
@@ -1059,6 +1061,7 @@ Boolean port_allow_io(ioport_t start, Bit16u size, int permission, Bit8u ormask,
 		 * port access was granted.
 		 * SIDOC_END_REMARK
 		 */
+		PRIV_SAVE_AREA
 		int devperm;
 
 		devrname=strrchr(device,'/');
@@ -1137,9 +1140,10 @@ Boolean port_allow_io(ioport_t start, Bit16u size, int permission, Bit8u ormask,
 int
 set_ioperm(int start, int size, int flag)
 {
+	PRIV_SAVE_AREA
 	int tmp;
 
-	if (!i_am_root || (start>0x3ff))
+	if (!can_do_root_stuff || (start>0x3ff))
 	    return -1;		/* don't bother */
 	if ((start+size)>0x3ff) size=0x400-start;
 
