@@ -30,9 +30,7 @@
 #ifdef __NetBSD__
 #include <errno.h>
 #endif
-#ifdef __linux__
-#include <asm/bitops.h>
-#endif
+#include "bitops.h"
 #if X_GRAPHICS
 #include <sys/mman.h>           /* root@sjoerd*/
 #endif
@@ -74,9 +72,7 @@
 #include "sound.h"
 #endif
 
-#ifdef NEW_KBD_CODE
 #include "keyb_server.h"
-#endif
 #include "keyboard.h"
 
 #include "dma.h"
@@ -147,6 +143,7 @@ int isATIport(int port)
 unsigned char
 inb(unsigned int port)
 {
+  PRIV_SAVE_AREA
 
   static unsigned int cga_r = 0;
   static unsigned char r;
@@ -175,9 +172,7 @@ inb(unsigned int port)
             ( ((port>=0x3c0) && (port<=0x3c1)) || /* root@zaphod attr ctrl */
               ((port>=0x3c4) && (port<=0x3c5)) || /* erik@zaphod sequencer */
               ((port>=0x3c6) && (port<=0x3c9)) || /* root@zaphod */
-#ifdef NEW_X_CODE
               ((port>=0x3d4) && (port<=0x3d5)) || /* sw: crt controller */
-#endif
               ((port>=0x3D0) && (port<=0x3DD)) ) )
     {
         r=VGA_emulate_inb((u_int)port);
@@ -239,11 +234,7 @@ inb(unsigned int port)
     r = keyb_io_read((u_int)port);
     return r;
   case 0x61:
-#ifndef NEW_KBD_CODE
-    r = keyb_io_read((u_int)port);
-#else
     r = spkr_io_read((u_int)port);
-#endif
     return r;
   case 0x70:
   case 0x71:
@@ -354,6 +345,7 @@ inb(unsigned int port)
 unsigned int
 inw(int port)
 {
+  PRIV_SAVE_AREA
   if (v_8514_base && ((port & 0x03fd) == v_8514_base) && (port & 0xfc00)) {
     int value;
 
@@ -382,6 +374,7 @@ inw(int port)
 unsigned int
 ind(int port)
 {
+  PRIV_SAVE_AREA
   int v;
   if (config.pci && (port >= 0xcf8) && (port < 0xd00)) {
     enter_priv_on();
@@ -398,6 +391,7 @@ ind(int port)
 void
 outb(unsigned int port, unsigned int byte)
 {
+  PRIV_SAVE_AREA
   static int lastport = 0;
   static unsigned int tmp = 0;
 
@@ -472,23 +466,15 @@ outb(unsigned int port, unsigned int byte)
       port_out(byte, port);
       set_ioperm(port, 1, 0);
     }
-#ifndef NEW_X_CODE
-    return;
-  }
-  else {
-#else
   }
   /* the above stuff should someday be moved to crtcemu.c -- sw */
   {
-#endif
 #if X_GRAPHICS
     if ( (config.X) &&
          ( ((port>=0x3c0) && (port<=0x3c1)) || /* root@zaphod attr ctrl */
            ((port>=0x3c4) && (port<=0x3c5)) || /* erik@zaphod sequencer */
            ((port>=0x3c6) && (port<=0x3c9)) || /* root@zaphod */
-#ifdef NEW_X_CODE
            ((port>=0x3d4) && (port<=0x3d5)) || /* sw: crt controller */
-#endif
            ((port>=0x3D0) && (port<=0x3DD)) ) )
       {
         VGA_emulate_outb(port, (unsigned char)byte);
@@ -579,11 +565,7 @@ outb(unsigned int port, unsigned int byte)
     keyb_io_write((u_int)port, byte);
     break;
   case 0x61:
-#ifndef NEW_KBD_CODE
-    keyb_io_write((u_int)port, byte);
-#else
     spkr_io_write((u_int)port, byte);
-#endif
     break;
   case 0x70:
   case 0x71:
@@ -664,6 +646,7 @@ outb(unsigned int port, unsigned int byte)
 void
 outw(unsigned int port, unsigned int value)
 {
+  PRIV_SAVE_AREA
   if (v_8514_base && ((port & 0x03fd) == v_8514_base) && (port & 0xfc00)) {
     enter_priv_on();
     iopl(3);
@@ -692,6 +675,7 @@ outw(unsigned int port, unsigned int value)
 void
 outd(unsigned int port, unsigned int value)
 {
+  PRIV_SAVE_AREA
   port &= 0xffff;
   if (config.pci && (port >= 0xcf8) && (port < 0xd00)) {
     enter_priv_on();
@@ -711,10 +695,11 @@ outd(unsigned int port, unsigned int value)
 
 Bit8u port_safe_inb(Bit32u port)
 {
+  PRIV_SAVE_AREA
   Bit8u res = 0;
 
   i_printf("PORT: safe_inb ");
-  if (i_am_root) {
+  if (can_do_root_stuff) {
     enter_priv_on();
     iopl(3);
     res = port_in(port);
@@ -725,7 +710,7 @@ Bit8u port_safe_inb(Bit32u port)
   else
     i_printf("want to ");
   i_printf("in(%lx)", port);
-  if (i_am_root)
+  if (can_do_root_stuff)
     i_printf(" = %hx", res);
   i_printf("\n");
   return res;
@@ -733,8 +718,9 @@ Bit8u port_safe_inb(Bit32u port)
 
 void port_safe_outb(Bit32u port, Bit8u byte)
 {
+  PRIV_SAVE_AREA
   i_printf("PORT: safe_outb ");
-  if (i_am_root) {
+  if (can_do_root_stuff) {
     enter_priv_on();
     iopl(3);
     port_out(byte, port);
@@ -748,10 +734,11 @@ void port_safe_outb(Bit32u port, Bit8u byte)
 
 Bit16u port_safe_inw(Bit32u port)
 {
+  PRIV_SAVE_AREA
   Bit16u res = 0;
 
   i_printf("PORT: safe_inw ");
-  if (i_am_root) {
+  if (can_do_root_stuff) {
     enter_priv_on();
     iopl(3);
     res = port_in_w(port);
@@ -761,7 +748,7 @@ Bit16u port_safe_inw(Bit32u port)
   else
     i_printf("want to ");
   i_printf("inw(%lx)", port);
-  if (i_am_root)
+  if (can_do_root_stuff)
     i_printf(" = %x", res);
   i_printf("\n");
   return res;
@@ -769,8 +756,9 @@ Bit16u port_safe_inw(Bit32u port)
 
 void port_safe_outw(Bit32u port, Bit16u word)
 {
+  PRIV_SAVE_AREA
   i_printf("PORT: safe_outw ");
-  if (i_am_root) {
+  if (can_do_root_stuff) {
     enter_priv_on();
     iopl(3);
     port_out_w(word, port);
@@ -796,9 +784,10 @@ int port_init(void)
 int
 set_ioperm(int start, int size, int flag)
 {
+    PRIV_SAVE_AREA
     int             tmp;
 
-    if (!i_am_root)
+    if (!can_do_root_stuff)
 	return -1;		/* don't bother */
 
 #ifdef X86_EMULATOR
