@@ -67,6 +67,7 @@ __asm__("___START___: jmp _emulate\n");
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
+#include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -129,9 +130,7 @@ __asm__("___START___: jmp _emulate\n");
 #endif
 #include "speaker.h"
 
-#ifdef NEW_KBD_CODE
 #include "keyb_clients.h"
-#endif
 
 #ifdef USE_SBEMU
 #include "sound.h"
@@ -149,6 +148,7 @@ extern void     vm86_GP_fault();
 extern void     config_init(int argc, char **argv);
 extern void	timer_int_engine(void);
 extern void	disk_open(struct disk *dp);
+extern void     print_version(void);
 
 extern void io_select_init(void);
 
@@ -156,11 +156,6 @@ extern void io_select_init(void);
 extern void treads_init(void);
 #endif
 
-static int      special_nowait = 0;
-
-#ifndef NEW_KBD_CODE
-extern void     keyboard_close(void);
-#endif
 
 
 void 
@@ -249,6 +244,7 @@ static inline void
 SIG_init()
 {
 #if defined(SIG) && defined(REQUIRES_VM86PLUS)
+    PRIV_SAVE_AREA
     /* Get in touch with Silly Interrupt Handling */
     if (config.sillyint) {
 	char            prio_table[] =
@@ -324,7 +320,6 @@ vm86plus_init(void)
 static inline void
 module_init(void)
 {
-    version_init();		/* Check the OS version */
     vm86plus_init();		/* emumodule support */
     SIG_init();			/* silly int generator support */
     memcheck_init();		/* lower 1M memory map support */
@@ -392,16 +387,8 @@ emulate(int argc, char **argv)
     changesegs();
 #endif
 
-    priv_init();  /* This must come first! */
+    srand(time(NULL));
 
-    /* the transposal of (config_|stdio_)init allows the addition of -o */
-    /* to specify a debug out filename, if you're wondering */
-
-    get_time_init();		/* debug can use CPUtime */
-    io_select_init();
-    port_init();		/* setup port structures, before config! */
-    config_init(argc, argv);	/* parse the commands & config file(s) */
-    get_time_init();
 #ifdef USE_THREADS
     treads_init();		/* init the threads system,
 				 * after this we will be thread0.
@@ -410,7 +397,28 @@ emulate(int argc, char **argv)
 				 * we capture it via 'exit'.
     				 */
 #endif
+    /* NOW! it is safe to touch the priv code.  */
+    priv_init();  /* This must come first! */
+
+    /* This has to come next:
+     * Parse dosemu.users _before_ any argument usage to catch
+     * a not allowed user playing with overruns and kick him out
+     * Additionally, we check a non-suid root condition, if dosemu.user
+     * says so and exit if needed.
+     */
+    parse_dosemu_users();
+
+    /* the transposal of (config_|stdio_)init allows the addition of -o */
+    /* to specify a debug out filename, if you're wondering */
+
+    get_time_init();		/* debug can use CPUtime */
+    io_select_init();
+    port_init();		/* setup port structures, before config! */
+    version_init();		/* Check the OS version */
+    config_init(argc, argv);	/* parse the commands & config file(s) */
+    get_time_init();
     stdio_init();		/* initialize stdio & open debug file */
+    print_version();            /* log version information */
     module_init();
     low_mem_init();		/* initialize the lower 1Meg */
     time_setting_init();	/* get the startup time */
@@ -424,7 +432,6 @@ emulate(int argc, char **argv)
 #endif
     memory_init();		/* initialize the memory contents */
     boot();			/* read the boot sector & get moving */
-    timer_interrupt_init();	/* start sending int 8h int signals */
 
     if (not_use_sigio)
 	k_printf("Atleast 1 NON-SIGIO file handle in use.\n");
@@ -437,6 +444,7 @@ emulate(int argc, char **argv)
 #ifdef USE_MHPDBG  
     mhp_debug(DBG_INIT, 0, 0);
 #endif
+    timer_interrupt_init();	/* start sending int 8h int signals */
 
     while (!fatalerr) {
 	++pic_vm86_count;
@@ -472,6 +480,7 @@ emulate(int argc, char **argv)
 
 #if 0    /* disable C-A-D until someone will fix it (if really needed) */
 extern void HMA_MAP(int);
+static int      special_nowait = 0;
 
 void
 dos_ctrl_alt_del(void)
@@ -479,11 +488,7 @@ dos_ctrl_alt_del(void)
     dbug_printf("DOS ctrl-alt-del requested.  Rebooting!\n");
     HMA_MAP(1);
     time_setting_init();
-#ifdef NEW_KBD_CODE   
     keyb_server_reset();
-#else   
-    keyboard_flags_init();
-#endif
     video_config_init();
     serial_init();
     mouse_init();
@@ -554,6 +559,12 @@ leavedos(int sig)
 #endif
     in_vm86 = 0;
 
+    /* try to notify dosdebug */
+    {
+      extern void mhp_exit_intercept(int errcode);
+      mhp_exit_intercept(sig);
+    }
+
     /* remove tmpdir */
     rmdir(tmpdir);
 
@@ -614,17 +625,10 @@ leavedos(int sig)
     video_close();
    
     g_printf("calling keyboard_close\n");
-#ifdef NEW_KBD_CODE 
     keyb_server_close();
     keyb_client_close();
 
 
-#else
-    fflush(stderr);
-    fflush(stdout);
-    keyboard_close();
-#endif
-   
     g_printf("calling shared memory exit\n");
     shared_memory_exit();
     g_printf("calling HMA exit\n");
