@@ -2,8 +2,8 @@
 
 extern void vga_init_avance(void);
 
-extern u_char avance_ext_video_port_in(int port);
-extern void avance_ext_video_port_out(u_char value, int port);
+extern u_char avance_ext_video_port_in(ioport_t port);
+extern void avance_ext_video_port_out(ioport_t port, u_char value);
 
 extern void avance_set_bank_read(u_char bank);
 extern void avance_set_bank_write(u_char bank);
