@@ -88,10 +88,6 @@ extern inline void console_update_cursor (int, int, int, int);
 static void set_dos_video ();
 void put_video_ram ();
 
-#ifdef NO_VC_NO_DEBUG
-static struct debug_flags save_debug_flags;
-#endif
-
 extern int
   dosemu_sigaction (int sig, struct sigaction *, struct sigaction *);
 static void SIGRELEASE_call (void);
@@ -161,7 +157,7 @@ void check_console(void) {
     minor = chkbuf.st_rdev & 0xff;
 
     /* console major num is 4, minor 64 is the first serial line */
-    if ((major == 4) && (minor < 64)) {
+    if (can_do_root_stuff && (major == 4) && (minor < 64)) {
        scr_state.console_no = minor;
        config.console=1;
     }
@@ -206,6 +202,7 @@ parent_close_mouse (void)
 static void
 parent_open_mouse (void)
 {
+  PRIV_SAVE_AREA
   if (mice->intdrv)
     {
       enter_priv_on(); /* The mouse may not be a resource everyone can open. */
@@ -238,8 +235,10 @@ acquire_vt (int sig, struct sigcontext_struct context)
   dos_has_vt = 1;
 
 #ifdef NO_VC_NO_DEBUG
-  d = save_debug_flags;
+  if (user_vc_switch) shut_debug = 0;
 #endif
+  flush_log();
+
   v_printf ("VID: Acquiring VC\n");
   forbid_switch ();
 #ifdef __NetBSD__
@@ -253,26 +252,12 @@ acquire_vt (int sig, struct sigcontext_struct context)
   scr_state.current = 1;
 }
 
-static inline void
-get_permissions (void)
-{
-  DOS_SYSCALL (set_ioperm (0x3b0, 0x3df - 0x3b0 + 1, 1));
-}
-
-static inline void
-giveup_permissions (void)
-{
-  DOS_SYSCALL (set_ioperm (0x3b0, 0x3df - 0x3b0 + 1, 0));
-}
-
 static void
 set_dos_video (void)
 {
   if (!config.vga)
     return;
 
-  v_printf ("Setting DOS video: gfx_mode: %d\n", gfx_mode);
-
   /* jesx */
   /* After all that fun up there, get permissions and save/restore states */
   if (video_initialized)
@@ -318,10 +303,6 @@ set_linux_video (void)
 static void
 SIGRELEASE_call (void)
 {
-#ifdef NO_VC_NO_DEBUG
-  save_debug_flags = d;
-#endif
-
   if (scr_state.current == 1)
     {
       v_printf ("VID: Releasing VC\n");
@@ -373,7 +354,7 @@ SIGRELEASE_call (void)
    /* we don't want any debug activity while VC is not active */
     flush_log();
 #ifdef NO_VC_NO_DEBUG
-    memset(&d, 0, sizeof(struct debug_flags));
+    if (user_vc_switch) shut_debug = 1;
 #endif
   }
 }
@@ -501,7 +482,6 @@ get_video_ram (int waitflag)
 
       close_kmem ();
       /* the code below is done by the video save/restore code */
-      get_permissions ();
     }
   else
     {
@@ -542,7 +522,6 @@ get_video_ram (int waitflag)
 	v_printf ("CONSOLE VIDEO address: %p %p %p\n", (void *) graph_mem,
 		  (void *) phys_text_base, (void *) PAGE_ADDR (READ_BYTE(BIOS_CURRENT_SCREEN_PAGE)));
 
-      get_permissions ();
       /* copy contents of page onto video RAM */
       memcpy ((caddr_t) PAGE_ADDR (READ_BYTE(BIOS_CURRENT_SCREEN_PAGE)), textbuf, TEXT_SIZE);
     }
@@ -595,7 +574,6 @@ put_video_ram (void)
 	  memcpy (scr_state.virt_address, putbuf, TEXT_SIZE);
 	}
 
-      giveup_permissions ();
       scr_state.mapped = 0;
     }
   else
@@ -634,6 +612,7 @@ set_process_control (void)
 void
 clear_process_control (void)
 {
+  PRIV_SAVE_AREA
   struct vt_mode vt_mode;
 
   vt_mode.mode = VT_AUTO;
@@ -652,6 +631,7 @@ static u_char kmem_open_count = 0;
 void
 open_kmem (void)
 {
+  PRIV_SAVE_AREA
   /* as I understad it, /dev/kmem is the kernel's view of memory,
      * and /dev/mem is the identity-mapped (i.e. physical addressed)
      * memory. Currently under Linux, both are the same.
