@@ -36,6 +36,21 @@ dosemu_sigaction(int sig, struct sigaction *, struct sigaction *);
 
 struct video_system *Video = NULL;
 
+/* I put Video_none here because I don't want to make a special file for such 
+ * a simple task -- Goga
+ */
+static int i_empty_void () {return 0;}
+static void v_empty_void () {}
+static int i_empty_3int (int type, int xsize, int ysize) {return 0;}
+
+struct video_system Video_none = {
+  0,		/* is_mapped */
+  i_empty_void,	/* init */
+  v_empty_void,	/* close */
+  i_empty_3int,	/* setmode */
+  i_empty_void,	/* update_screen */
+  v_empty_void	/* update_cursor */
+};
 
 /* 
  * DANG_BEGIN_FUNCTION video_init
@@ -76,6 +91,10 @@ int video_init()
      v_printf("VID: Video set to Video_hgc\n");
      Video=&Video_hgc;
   }
+  else if (config.cardtype == CARD_NONE) {
+     v_printf("VID: Video set to Video_none");
+     Video=&Video_none;
+  }
   else {
      v_printf("VID: Video set to Video_term\n");
      Video=&Video_term;       /* ansi or ncurses */
@@ -346,36 +365,6 @@ video_config_init(void) {
       break;
     }
   }
-  WRITE_BYTE(BIOS_CURRENT_SCREEN_PAGE, 0x0);	/* Current Screen Page */
-  video_page = 0;
-  screen_mask = 1 << (((int)phys_text_base-0xA0000)/4096);
-  screen_adr = SCREEN_ADR(0);
-  WRITE_WORD(BIOS_CURSOR_SHAPE, (configuration&MDA_CONF_SCREEN_MODE)?0x0A0B:0x0607);
-  
-  /* This is needed in the video stuff. Grabbed from boot(). */
-  if ((configuration & MDA_CONF_SCREEN_MODE) == MDA_CONF_SCREEN_MODE)
-    WRITE_WORD(BIOS_VIDEO_PORT, 0x3b4);	/* base port of CRTC - IMPORTANT! */
-  else
-    WRITE_WORD(BIOS_VIDEO_PORT, 0x3d4);	/* base port of CRTC - IMPORTANT! */
-
-  WRITE_BYTE(BIOS_VDU_CONTROL, 9);	/* current 3x8 (x=b or d) value */
-
-  WRITE_BYTE(BIOS_VIDEO_MODE, video_mode); /* video mode */
-  set_video_bios_size();
-  WRITE_WORD(BIOS_VIDEO_MEMORY_ADDRESS, 0);/* offset of current page in buffer */
-
-  WRITE_WORD(BIOS_FONT_HEIGHT, 16);
-  
-  /* XXX - these are the values for VGA color!
-     should reflect the real display hardware. */
-  WRITE_BYTE(BIOS_VIDEO_INFO_0, 0x60);
-  WRITE_BYTE(BIOS_VIDEO_INFO_1, 0xF9);
-  WRITE_BYTE(BIOS_VIDEO_INFO_2, 0x51);
-  WRITE_BYTE(BIOS_VIDEO_COMBO, video_combo);
-
-  WRITE_DWORD(BIOS_VIDEO_SAVEPTR, 0);		/* pointer to video table */
-
-#ifdef NEW_KBD_CODE
   if (!config.console) {
      /* NOTE: BIG FAT WARNING !!!
       *       without this you will reproduceable KILL LINUX
@@ -399,12 +388,39 @@ video_config_init(void) {
      if (config.speaker == SPKR_NATIVE)
         config.speaker = SPKR_EMULATED;
   }
+  video_page = 0;
+  screen_mask = 1 << (((int)phys_text_base-0xA0000)/4096);
+  screen_adr = SCREEN_ADR(0);
+  if (!config.vga) {
+    WRITE_BYTE(BIOS_CURRENT_SCREEN_PAGE, 0x0);	/* Current Screen Page */
+    WRITE_WORD(BIOS_CURSOR_SHAPE, (configuration&MDA_CONF_SCREEN_MODE)?0x0A0B:0x0607);
+    
+    /* This is needed in the video stuff. Grabbed from boot(). */
+    if ((configuration & MDA_CONF_SCREEN_MODE) == MDA_CONF_SCREEN_MODE)
+      WRITE_WORD(BIOS_VIDEO_PORT, 0x3b4);	/* base port of CRTC - IMPORTANT! */
+    else
+      WRITE_WORD(BIOS_VIDEO_PORT, 0x3d4);	/* base port of CRTC - IMPORTANT! */
+    
+    WRITE_BYTE(BIOS_VDU_CONTROL, 9);	/* current 3x8 (x=b or d) value */
+    
+    WRITE_BYTE(BIOS_VIDEO_MODE, video_mode); /* video mode */
+    set_video_bios_size();
+    WRITE_WORD(BIOS_VIDEO_MEMORY_ADDRESS, 0);/* offset of current page in buffer */
+    
+    WRITE_WORD(BIOS_FONT_HEIGHT, 16);
+    
+    /* XXX - these are the values for VGA color!
+       should reflect the real display hardware. */
+    WRITE_BYTE(BIOS_VIDEO_INFO_0, 0x60);
+    WRITE_BYTE(BIOS_VIDEO_INFO_1, 0xF9);
+    WRITE_BYTE(BIOS_VIDEO_INFO_2, 0x51);
+    WRITE_BYTE(BIOS_VIDEO_COMBO, video_combo);
+    
+    WRITE_DWORD(BIOS_VIDEO_SAVEPTR, 0);		/* pointer to video table */
+  }
+    
   if (config.console_video && !config.usesX)
     set_process_control();
-#else
-  if ((config.console_keyb || config.console_video) && !config.usesX)
-    set_process_control();
-#endif
 
   if (config.console_video) {
     set_console_video();
