@@ -60,7 +60,8 @@
 #define bios_24_hours_flag              (*(unsigned char  *) 0x470)
 #define bios_keyboard_flags             (*(unsigned char  *) 0x471)
 #define bios_ctrl_alt_del_flag          (*(unsigned short *) 0x472)
-/* 0x474-0x477 is reserved */
+#define bios_harddisk_count		(*(unsigned short *) 0x475)
+/* 0x474, 0x476, 0x477 is reserved */
 #define bios_lpt1_timeout               (*(unsigned char  *) 0x478)
 #define bios_lpt2_timeout               (*(unsigned char  *) 0x479)
 #define bios_lpt3_timeout               (*(unsigned char  *) 0x47a)
@@ -88,16 +89,12 @@
 
 
 
-typedef unsigned char ubyte_t;
-typedef unsigned short uword_t;
-typedef unsigned long udword_t;
-
-#define READ_BYTE(addr)                 (*(ubyte_t *) (addr))
-#define WRITE_BYTE(addr, val)           (*(ubyte_t *) (addr) = (val) )
-#define READ_WORD(addr)                 (*(uword_t *) (addr))
-#define WRITE_WORD(addr, val)           (*(uword_t *) (addr) = (val) )
-#define READ_DWORD(addr)                (*(udword_t *) (addr))
-#define WRITE_DWORD(addr, val)          (*(udword_t *) (addr) = (val) )
+#define READ_BYTE(addr)                 (*(Bit8u *) (addr))
+#define WRITE_BYTE(addr, val)           (*(Bit8u *) (addr) = (val) )
+#define READ_WORD(addr)                 (*(Bit16u *) (addr))
+#define WRITE_WORD(addr, val)           (*(Bit16u *) (addr) = (val) )
+#define READ_DWORD(addr)                (*(Bit32u *) (addr))
+#define WRITE_DWORD(addr, val)          (*(Bit32u *) (addr) = (val) )
 
 #define MEMCPY_2UNIX(unix_addr, dos_addr, n) \
 	memcpy((unix_addr), (Bit8u *)(dos_addr), (n))
@@ -133,11 +130,7 @@ typedef unsigned long udword_t;
 /* #define bios_expansion_memory_size      (*(unsigned int   *) 0x415) */
 #define BIOS_KEYBOARD_STATE             0x417
 #define BIOS_KEYBOARD_FLAGS1            BIOS_KEYBOARD_STATE
-#ifdef NEW_KBD_CODE
 #define BIOS_KEYBOARD_FLAGS2            0x418
-#else
-#define BIOS_KEYBOARD_LEDS              0x418
-#endif
 #define BIOS_KEYBOARD_TOKEN             0x419
 /* used for keyboard input with Alt-Number */
 #define BIOS_KEYBOARD_BUFFER_HEAD       0x41a
@@ -178,7 +171,8 @@ typedef unsigned long udword_t;
 #define BIOS_24_HOURS_FLAG              0x470
 #define BIOS_KEYBOARD_FLAGS             0x471
 #define BIOS_CTRL_ALT_DEL_FLAG          0x472
-/* 0x474-0x477 is reserved */
+#define BIOS_HARDDISK_COUNT		0x475
+/* 0x474, 0x476, 0x477 is reserved */
 #define BIOS_LPT1_TIMEOUT               0x478
 #define BIOS_LPT2_TIMEOUT               0x479
 #define BIOS_LPT3_TIMEOUT               0x47a
@@ -198,12 +192,8 @@ typedef unsigned long udword_t;
 #define BIOS_VIDEO_INFO_2               0x489
 #define BIOS_VIDEO_COMBO                0x48a
 
-#ifdef NEW_KBD_CODE
 #define BIOS_KEYBOARD_FLAGS3            0x496
 #define BIOS_KEYBOARD_LEDS              0x497
-#else
-#define BIOS_KEYBOARD_FLAGS2            0x496
-#endif
 #define BIOS_PRINT_SCREEN_FLAG          0x500
 
 #define BIOS_VIDEO_SAVEPTR              0x4a8
@@ -226,7 +216,7 @@ typedef unsigned long udword_t;
  * with num > 4
  */
 #define CONF_NSER(c,num)	{c&=~(BIT(9)|BIT(10)|BIT(11)); c|=(num<<9);}
-#define CONF_NLPT(c,num) 	{c&=~(BIT(14)|BIT(14)); c|=(num<<14);}
+#define CONF_NLPT(c,num) 	{c&=~(BIT(14)|BIT(15)); c|=(num<<14);}
 #define CONF_NFLOP(c,num) 	{c&=~(CONF_FLOP|BIT(6)|BIT(7)); \
 				   if (num) c|=((num-1)<<6)|CONF_FLOP;}
 
@@ -243,6 +233,10 @@ void            INT09_dummy_start();
 void            INT09_dummy_end();
 void            INT08_dummy_start();
 void            INT08_dummy_end();
+#ifdef NEW_CMOS
+void            INT70_dummy_start();
+void            INT70_dummy_end();
+#endif
 void            DPMI_dummy_start();
 void            DPMI_dummy_end();
 void            DPMI_dpmi_init();
