@@ -2,7 +2,9 @@
 #define DOSEMU_DEBUG_H
 
 #include <stdio.h>
+#include <stdarg.h>
 #include "config.h"
+#include "extern.h"
 
 #if __GNUC__ >= 2
 # define FORMAT(T,A,B)  __attribute__((format(T,A,B)))
@@ -65,6 +67,8 @@ void p_dos_str(char *,...) FORMAT(printf, 1, 2);
 
 extern FILE *dbg_fd;
 
+EXTERN int shut_debug INIT(0);
+
 #ifdef DEBUG_LITE
 #define ifprintf(flg,fmt,a...)	do{ if (flg) log_printf(0x10000|__LINE__,fmt,##a); }while(0)
 #else
@@ -76,6 +80,7 @@ extern FILE *dbg_fd;
 
 /* unconditional message into debug log and stderr */
 void error(const char *fmt, ...);
+void verror(const char *fmt, va_list args);
 
 #define flush_log()		{ if (dbg_fd) log_printf(-1, "\n"); }
 
