@@ -9,7 +9,7 @@
 #include <features.h>
 #include <sys/types.h>
 #include <signal.h> 
-#if __GLIBC__ > 1
+#if __GLIBC__ == 2 && __GLIBC_MINOR__ == 0
 #include <sigcontext.h>
 #endif
 #include "config.h"
@@ -55,20 +55,6 @@ EXTERN int terminal_pipe;
 EXTERN int terminal_fd INIT(-1);
 EXTERN int running_kversion INIT(0);
 
-/* set to one if runing setuid as root */
-#if 1
-#define i_am_root 1  /* neadless to have it variable, it will be allways 1
-                      * If dosemu isn't suid root, it will terminate
-                      */
-#else
-EXTERN int i_am_root INIT(0);
-#endif
-
-/* set, if dosemu was started from a 'root login',
- * unset, if dosemu was started from a 'user login' (running suid root)
- */
-EXTERN int under_root_login INIT(0);
-
 EXTERN char *cstack[16384];
 
 /* this is DEBUGGING code! */
@@ -99,9 +85,6 @@ EXTERN char *cl,		/* clear screen */
 
 /* the fd for the keyboard */ 
 EXTERN int console_fd INIT(-1);
-#ifndef NEW_KBD_CODE
-  #define kbd_fd console_fd
-#endif
 
 /* the file descriptor for /dev/mem when mmap'ing the video mem */
 EXTERN int mem_fd INIT(-1);
@@ -160,21 +143,6 @@ EXTERN u_char in_ioctl INIT(0);
 EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
 
 
-/* int 11h config single bit tests */
-#define CONF_FLOP	BIT(0)
-#define CONF_MATHCO	BIT(1)
-#define CONF_MOUSE	BIT(2)
-#define CONF_DMA	BIT(8)
-#define CONF_GAME	BIT(12)
-
-/* don't use CONF_NSER with num > 4, CONF_NLPT with num > 3, CONF_NFLOP
- * with num > 4
- */
-#define CONF_NSER(c,num)	{c&=~(BIT(9)|BIT(10)|BIT(11)); c|=(num<<9);}
-#define CONF_NLPT(c,num) 	{c&=~(BIT(14)|BIT(14)); c|=(num<<14);}
-#define CONF_NFLOP(c,num) 	{c&=~(CONF_FLOP|BIT(6)|BIT(7)); \
-				   if (num) c|=((num-1)<<6)|CONF_FLOP;}
-
 /* this macro can be safely wrapped around a system call with no side
  * effects; using a feature of GCC, it returns the same value as the
  * function call argument inside.
@@ -278,6 +246,8 @@ typedef struct vesamode_type_struct {
        char    *X_title;                /* X window title */
        char    *X_icon_name;
        char    *X_font;
+       char    *X_mgrab_key;		/* KeySym name to activate mouse grab */
+					/* "" turns it of, NULL gives the default ("Home") */
        int     X_blinkrate;
        int     X_sharecmap;
        int     X_mitshm;                /* use MIT SHM extension */
@@ -306,6 +276,7 @@ typedef struct vesamode_type_struct {
        int     realcpu;
        boolean mathco, smp;
        boolean ipxsup;
+       boolean vnet;
        boolean keybint;
        boolean dosbanner;
        boolean allowvideoportaccess;
@@ -350,11 +321,8 @@ typedef struct vesamode_type_struct {
        int sillyint;            /* IRQ numbers for Silly Interrupt Generator 
        				   (bitmask, bit3..15 ==> IRQ3 .. IRQ15) */
 
-       int keyboard;
-       unsigned char *key_map;     /* pointer to the correct keyboard-map */
-       unsigned char *shift_map;
-       unsigned char *alt_map;
-       unsigned char *num_table;
+       struct keytable_entry *keytable;
+
        unsigned short detach;
        unsigned char *debugout;
        unsigned char *pre_stroke;  /* pointer to keyboard pre strokes */
@@ -380,32 +348,6 @@ config_t;
 #define SPKR_NATIVE	1
 #define SPKR_EMULATED	2
 
-#define KEYB_FINNISH           0
-#define KEYB_FINNISH_LATIN1    1
-#define KEYB_US                2
-#define KEYB_UK                3
-#define KEYB_DE                4
-#define KEYB_DE_LATIN1         5
-#define KEYB_FR                6
-#define KEYB_FR_LATIN1         7
-#define KEYB_DK                8
-#define KEYB_DK_LATIN1         9
-#define KEYB_DVORAK           10
-#define KEYB_SG               11
-#define KEYB_SG_LATIN1        12
-#define KEYB_NO               13
-#define KEYB_NO_LATIN1        14
-#define KEYB_SF               15
-#define KEYB_SF_LATIN1        16
-#define KEYB_ES               17
-#define KEYB_ES_LATIN1        18
-#define KEYB_BE               19
-#define KEYB_PO               20
-#define KEYB_IT               21
-#define KEYB_SW		      22
-#define KEYB_HU		      23
-#define KEYB_HU_CWI	      24
-#define KEYB_HU_LATIN2	      25
 /*
  * Right now, dosemu only supports two serial ports.
  */
@@ -429,7 +371,12 @@ EXTERN int write_port(unsigned int, unsigned short);
 EXTERN __inline__ void parent_nextscan(void);
 EXTERN __inline__ void disk_close(void);
 EXTERN void cpu_setup(void);
-EXTERN __inline__ void run_int(int);
+EXTERN void real_run_int(int);
+#ifdef USE_NEW_INT
+  #define run_int do_int
+#else
+  #define run_int real_run_int
+#endif
 EXTERN int mfs_redirector(void);
 EXTERN void int10(void);
 EXTERN void int13(u_char);
@@ -443,7 +390,7 @@ EXTERN void floppy_tick(void);
 EXTERN void open_kmem(void);
 EXTERN void close_kmem(void);
 EXTERN void CloseNetworkLink(int);
-EXTERN int parse_config(char *);
+EXTERN int parse_config(char *, char *);
 EXTERN void disk_init(void);
 EXTERN void serial_init(void);
 EXTERN void close_all_printers(void);
@@ -455,7 +402,7 @@ EXTERN int mfs_inte6(void);
 EXTERN void pkt_helper(void);
 EXTERN short pop_word(struct vm86_regs *);
 EXTERN void ems_init(void);
-EXTERN int GetDebugFlagsHelper(char *);
+EXTERN int GetDebugFlagsHelper(char *, int);
 EXTERN int SetDebugFlagsHelper(char *);
 EXTERN void leavedos(int) NORETURN;
 EXTERN void add_to_io_select(int, unsigned char);
@@ -495,14 +442,20 @@ EXTERN void sigio(int, int, struct sigcontext *);
  * description:
  *  All signals that wish to be handled properly in context with the
  * execution of vm86() mode, and signals that wish to use non-reentrant
- * functions should add themselves to the SIGNALS_THAT_QUEUE define and
- * use SETQSIG(). To that end they will also need to be set up in an
+ * functions should add themselves to the ADDSET_SIGNALS_THAT_QUEUE define
+ * and use SETQSIG(). To that end they will also need to be set up in an
  * order such as SIGIO.
  *
  * DANG_END_FUNCTION
  *
  */
-#define SIGNALS_THAT_QUEUE SIGIO|SIGALRM|SIG_RELEASE|SIG_ACQUIRE
+#define ADDSET_SIGNALS_THAT_QUEUE(x) \
+do { \
+       sigaddset(x, SIGIO); \
+       sigaddset(x, SIG_TIME); \
+       sigaddset(x, SIG_RELEASE); \
+       sigaddset(x, SIG_ACQUIRE); \
+} while(0)
 
 #ifdef __NetBSD__
 #define SignalHandler sig_t
@@ -511,7 +464,7 @@ EXTERN void sigio(int, int, struct sigcontext *);
 #define NEWSETQSIG(sig, fun)	sa.sa_handler = (__sighandler_t)fun; \
 					sa.sa_flags = SA_RESTART|SA_ONSTACK ; \
 					sigemptyset(&sa.sa_mask); \
-					sigaddset(&sa.sa_mask, SIGNALS_THAT_QUEUE); \
+					ADDSET_SIGNALS_THAT_QUEUE(&sa.sa_mask); \
 					sigaction(sig, &sa, NULL);
 
 #define SETSIG(sig, fun)	sa.sa_handler = (__sighandler_t)fun; \
@@ -538,7 +491,7 @@ EXTERN void sigio(int, int, struct sigcontext *);
 			(void (*)()) (((unsigned int)(cstack) + sizeof(cstack) - 4) & ~3); \
 					sa.sa_flags = SA_RESTART ; \
 					sigemptyset(&sa.sa_mask); \
-					sigaddset(&sa.sa_mask, SIGNALS_THAT_QUEUE); \
+					ADDSET_SIGNALS_THAT_QUEUE(&sa.sa_mask); \
 					dosemu_sigaction(sig, &sa, NULL);
 
 #define SETSIG(sig, fun)	sa.sa_handler = (SignalHandler)fun; \
